/* DEBUG CLASS */

void Debug::init() {
  if (Debug::LEVEL > 0) {
    Serial.begin(Debug::BAUD_RATE);
    delay(Main::SLEEP_DELAY);
  }
}

void Debug::print(String message) {
  if (Debug::LEVEL == 3) {
    Serial.print(message);
  }
}

void Debug::printLine(String message) {
  if (Debug::LEVEL == 3) {
    Serial.println(message);
  }
}

void Debug::printStart(String message) {
  if (Debug::LEVEL == 3) {
    Serial.print(message + ": ");
  }
}

void Debug::printSeparationLine(String message) {
  if (Debug::LEVEL == 3) {
    Serial.println();
    Serial.println(message);
    Serial.println();
  }
}

void Debug::printWarning(String message) {
  if (Debug::LEVEL == 3) {
    Serial.println("WARNING");
  }
  if (Debug::LEVEL > 1) {
    Serial.println("WARNING: " + message);
  }
}

void Debug::printError(String message) {
  if (Debug::LEVEL == 3) {
    Serial.println("ERROR");
  }
  if (Debug::LEVEL > 0) {
    Serial.println("ERROR: " + message);
  }
}

void Debug::printNull() {
  if (Debug::LEVEL == 3) {
    Serial.println("NULL");
  }
}

void Debug::printBoolean(bool value) {
  if (Debug::LEVEL == 3) {
    if (value) {
      Serial.println("TRUE");
    } else {
      Serial.println("FALSE");
    }
  }
}

void Debug::printSetupInfo() {
  if (Debug::LEVEL == 3) {
    FSInfo fsInfo;
    LittleFS.info(fsInfo);
    Serial.println("Setup info:");
    Serial.println("Flash chip ID: " + String(ESP.getFlashChipId()));
    Serial.println("Flash chip size: " + String(ESP.getFlashChipRealSize()) + " B");
    Serial.println("Flash chip speed: " + String(ESP.getFlashChipSpeed()) + " Hz");
    Serial.println("Flash chip mode: " + String(ESP.getFlashChipMode()));
    Serial.println("Free sketch space: " + String(ESP.getFreeSketchSpace()) + " B");
    Serial.println("Free heap: " + String(ESP.getFreeHeap()) + " B");
    Serial.println("Boot mode: " + String(ESP.getBootMode()));
    Serial.println("Boot version: " + String(ESP.getBootVersion()));
    Serial.println("Core version: " + String(ESP.getCoreVersion()));
    Serial.println("SDK version: " + String(ESP.getSdkVersion()));
    Serial.println("Reset reason: " + String(ESP.getResetReason()));
    Serial.println("Reset info: " + String(ESP.getResetInfo()));
    Serial.println("File system total size: " + String(fsInfo.totalBytes) + " B");
    Serial.println("File system used size: " + String(fsInfo.usedBytes) + " B");
    Serial.println("File system free size: " + String(fsInfo.totalBytes - fsInfo.usedBytes) + " B");
    Serial.println("Firmware version: " + FIRMWARE_VERSION);
    Serial.println("Setup mode: " + String(Main::SERVER_MODE ? "SERVER" : "CLIENT"));
    Serial.println("Button pin: " + String(Pin::BUTTON));
    Serial.println("Relay pin: " + String(Pin::RELAY));
    Serial.println("Led pin: " + String(Pin::LED));
    Serial.println("External button pin: " + String(Pin::EXT_BUTTON));
    Serial.println("External LED pin: " + String(Pin::EXT_LED));
    Serial.println("Temperature sensors pin: " + String(Pin::TEMP_SENSORS));
    Serial.println("LED 1 pin: " + String(Pin::LED_1));
    Serial.println("LED 2 pin: " + String(Pin::LED_2));
    Serial.println("LED 3 pin: " + String(Pin::LED_3));
    Serial.println("Device ID: '" + Data::DEVICE_ID + "'");
    Serial.println("Device name: '" + Data::DEVICE_NAME + "'");
    Serial.println("Device token: '" + Data::DEVICE_TOKEN + "'");
    Serial.println("REST API URL: '" + Network::REST_API_URL + "'");
  }
}

void Debug::printJsonData(String message) {
  if (Debug::LEVEL == 3) {
    Serial.println(message + ": (" + measureJson(JSON_DATA) + " B)");
    String jsonText = "";
    serializeJsonPretty(JSON_DATA, jsonText);
    Serial.println(jsonText);
  }
}

void Debug::printNetworksData(String message) {
  if (Debug::LEVEL == 3) {
    Serial.println(message + ":");
    Serial.println("Number of networks: " + String(NETWORKS_CNT));
    for (int i = 0; i < NETWORKS_CNT; i++) {
      Serial.print("Key: " + String(i));
      Serial.print(", SSID: '" + NETWORKS[i].ssid + "'");
      Serial.print(", password: '" + NETWORKS[i].password + "'");
      Serial.println();
    }
  }
}

void Debug::printScannedNetworksData(String message) {
  if (Debug::LEVEL == 3) {
    Serial.println(message + ":");
    Serial.println("Number of scanned networks: " + String(SCANNED_NETWORKS_CNT));
    for (int i = 0; i < SCANNED_NETWORKS_CNT; i++) {
      Serial.print("Key: " + String(i));
      Serial.print(", SSID: '" + SCANNED_NETWORKS[i].ssid + "'");
      Serial.print(", RSSI: " + String(SCANNED_NETWORKS[i].rssi) + " dBm");
      Serial.print(", mac: " + SCANNED_NETWORKS[i].bssid);
      Serial.print(", ch: " + String(SCANNED_NETWORKS[i].channel));
      Serial.print(", enc: " + String(SCANNED_NETWORKS[i].encryptionType));
      Serial.println();
    }
  }
}
