/* PIN CLASS */

void Pin::init() {
  Debug::printStart("Configuring pins");
  if (Pin::BUTTON > Pin::NONE) {
    pinMode(Pin::BUTTON, INPUT_PULLUP);
    attachInterrupt(digitalPinToInterrupt(Pin::BUTTON), Pin::buttonInterrupt, CHANGE);
  }
  if (Pin::RELAY > Pin::NONE) {
    pinMode(Pin::RELAY, OUTPUT);
  }
  if (Pin::LED > Pin::NONE) {
    pinMode(Pin::LED, OUTPUT);
  }
  if (Pin::EXT_BUTTON > Pin::NONE) {
    pinMode(Pin::EXT_BUTTON, INPUT_PULLUP);
    attachInterrupt(digitalPinToInterrupt(Pin::EXT_BUTTON), Pin::extButtonInterrupt, CHANGE);
  }
  if (Pin::EXT_LED > Pin::NONE) {
    pinMode(Pin::EXT_LED, OUTPUT);
  }
  if (Pin::LED_1 > Pin::NONE) {
    pinMode(Pin::LED_1, OUTPUT);
    analogWrite(Pin::LED_1, Pin::LED_1_VALUE);
  }
  if (Pin::LED_2 > Pin::NONE) {
    pinMode(Pin::LED_2, OUTPUT);
    analogWrite(Pin::LED_2, Pin::LED_2_VALUE);
  }
  if (Pin::LED_3 > Pin::NONE) {
    pinMode(Pin::LED_3, OUTPUT);
    analogWrite(Pin::LED_3, Pin::LED_3_VALUE);
  }
  if (Pin::TEMP_SENSORS > Pin::NONE) {
    TEMPERATURE_SENSORS.setOneWire(new OneWire(Pin::TEMP_SENSORS));
    TEMPERATURE_SENSORS.begin();
  }
  Debug::printLine("OK");
}

void Pin::changeRelayState(bool state) {
  if (Pin::RELAY > Pin::NONE) {
    if (state) {
      // Debug::printStart("Changing relay state to ON");
      digitalWrite(Pin::RELAY, HIGH);
      Pin::RELAY_STATE = true;
      // Debug::printLine("OK");
    } else {
      // Debug::printStart("Changing relay state to OFF");
      digitalWrite(Pin::RELAY, LOW);
      Pin::RELAY_STATE = false;
      // Debug::printLine("OK");
    }
  }
}

void Pin::pressRelay() {
  if (Pin::RELAY > Pin::NONE && Pin::RELAY_BUTTON_MODE) {
    Debug::printStart("Changing relay state to ON (button mode)");
    digitalWrite(Pin::RELAY, HIGH);
    Debug::printLine("OK");
    Main::sleep(Pin::RELAY_MAX_STATE_DELAY);
    Debug::printStart("Changing relay state to OFF (button mode)");
    digitalWrite(Pin::RELAY, LOW);
    Debug::printLine("OK");
    Pin::RELAY_STATE = true;
  }
}

void Pin::changeLedValues(int led1value, int led2value, int led3value) {
  Debug::printStart("Changing LED states to: (" + String(led1value) + ", " + String(led2value) + ", " + String(led3value) + ")");
  if (Pin::LED_1 > Pin::NONE) {
    Pin::LED_1_VALUE = led1value;
    analogWrite(Pin::LED_1, Pin::LED_1_VALUE);
  }
  if (Pin::LED_2 > Pin::NONE) {
    Pin::LED_2_VALUE = led2value;
    analogWrite(Pin::LED_2, Pin::LED_2_VALUE);
  }
  if (Pin::LED_3 > Pin::NONE) {
    Pin::LED_3_VALUE = led3value;
    analogWrite(Pin::LED_3, Pin::LED_3_VALUE);
  }
  Debug::printLine("OK");
}

void Pin::protectLongRelayStateChange() {
  if (Pin::RELAY > Pin::NONE && !Pin::RELAY_BUTTON_MODE) {
    unsigned long currentTime = millis();
    int remRelayStateTime = Pin::RELAY_MAX_STATE_DELAY - (currentTime - Network::LAST_SUCCESS_REQUEST_TIME);
    Debug::printLine("Current time: " + String(currentTime) + " ms");
    Debug::printLine("Remaining relay state time: " + String(remRelayStateTime) + " ms");
    Debug::printLine("Maximum relay state time: " + String(Pin::RELAY_MAX_STATE_DELAY) + " ms");
    if (remRelayStateTime <= 0) {
      Debug::printWarning("RELAY PROTECT: Reseting relay to default state.");
      Pin::changeRelayState(Pin::RELAY_DEFAULT_STATE);
    }
  }
}

void Pin::setLedOn() {
  if (Pin::LED >= 0) {
    Debug::printStart("Turning LED ON");
    digitalWrite(Pin::LED, LOW);
    Debug::printLine("OK");
  }
  if (Pin::EXT_LED >= 0) {
    Debug::printStart("Turning external LED ON");
    digitalWrite(Pin::EXT_LED, HIGH);
    Debug::printLine("OK");
  }
}

void Pin::setLedOff() {
  if (Pin::LED >= 0) {
    Debug::printStart("Turning LED OFF");
    digitalWrite(Pin::LED, HIGH);
    Debug::printLine("OK");
  }
  if (Pin::EXT_LED >= 0) {
    Debug::printStart("Turning external LED OFF");
    digitalWrite(Pin::EXT_LED, LOW);
    Debug::printLine("OK");
  }
}

void Pin::ledBlink(int cnt) {
  if (Pin::LED >= 0 || Pin::EXT_LED >= 0) {
    Debug::printLine("LED blink: " + String(cnt));
    for (int i = 0; i < cnt; i++) {
      if (i != 0) {
        delay(Pin::LED_BLINK_DELAY);
      }
      Pin::setLedOn();
      delay(Pin::LED_BLINK_ON_TIME);
      Pin::setLedOff();
    }
  }
}

void ICACHE_RAM_ATTR Pin::buttonInterrupt() {
  if (digitalRead(Pin::BUTTON) == HIGH) {
    Pin::buttonRiseInterrupt();
  }
  Pin::LAST_BUTTON_FALL_TIME = millis();
}

void ICACHE_RAM_ATTR Pin::extButtonInterrupt() {
  if (digitalRead(Pin::EXT_BUTTON) == HIGH) {
    Pin::buttonRiseInterrupt();
  }
  Pin::LAST_BUTTON_FALL_TIME = millis();
}

void Pin::buttonRiseInterrupt() {
  unsigned long currentTime = millis();
  int diffTime = currentTime - Pin::LAST_BUTTON_FALL_TIME;
  if (diffTime > Pin::BUTTON_MIN_DELAY) {
    Debug::printLine("---------- BUTTON PRESSED FOR: " + String(diffTime) + " ms");
    if (diffTime > Pin::BUTTON_LONG_PRESS_DELAY) {
      Main::changeModuleMode(!Main::SERVER_MODE);
    } else {
      Pin::changeRelayState(!Pin::RELAY_STATE);
      Main::BREAK_SLEEP = true;
    }
  }
}
