/* DATA CLASS */

bool Data::init() {
  //Debug::printStart("Mounting file system");
  if (!LittleFS.begin()) {
    //Debug::printError("MOUNT FS: Cannot mount file system.");
    return false;
  }
  //Debug::printLine("OK");
  return true;
}

bool Data::clearJson() {
  //Debug::printStart("Clearing JSON data");
  DeserializationError error = deserializeJson(JSON_DATA, "{}");
  if (error) {
    //Debug::printError("CLEAR JSON: Cannot clear JSON data.");
    //Debug::printError(error.f_str());
    return false;
  }
  //Debug::printLine("OK");
  return true;
}

bool Data::loadDefault() {
  Data::loadJsonFromFile(Data::DEFAULT_FILE);
  Data::loadStringFromJson(Data::DEVICE_ID, "deviceId");
  Data::DEVICE_NAME = Data::DEVICE_NAME_PREFIX + Data::DEVICE_ID;
  Data::loadStringFromJson(Data::DEVICE_TOKEN, "deviceToken");
  Network::REST_API_URL = Network::REST_API_BASE_URL + Data::DEVICE_TOKEN;
  return true;
}

bool Data::loadVariables() {
  Data::loadJsonFromFile(Data::VARIABLES_FILE);
  return Data::loadVariablesFromJson();
}

bool Data::loadConfig() {
  Data::loadJsonFromFile(Data::CONFIG_FILE);
  return Data::loadConfigFromJson();
}

bool Data::loadSettings() {
  Data::loadJsonFromFile(Data::SETTINGS_FILE);
  return Data::loadSettingsFromJson();
}

bool Data::loadNetworks() {
  Data::loadJsonFromFile(Data::NETWORKS_FILE);
  return Data::loadNetworksFromJson("networks");
}

bool Data::saveVariables() {
  Data::clearJson();
  Data::saveVariablesToJson();
  return Data::saveJsonToFile(Data::VARIABLES_FILE);
}

bool Data::saveConfig() {
  Data::clearJson();
  Data::saveConfigToJson();
  return Data::saveJsonToFile(Data::CONFIG_FILE);
}

bool Data::saveSettings() {
  Data::clearJson();
  Data::saveSettingsToJson();
  return Data::saveJsonToFile(Data::SETTINGS_FILE);
}

bool Data::saveNetworks() {
  Data::clearJson();
  Data::saveNetworksToJson("networks");
  return Data::saveJsonToFile(Data::NETWORKS_FILE);
}

bool Data::loadBooleanFromJson(bool &variable, String key) {
  //Debug::printStart("Loading boolean value from JSON '" + key + "' key");
  if (JSON_DATA.containsKey(key)) {
    bool value = JSON_DATA[key].as<bool>();
    variable = value;
    //Debug::printBoolean(value);
    return true;
  }
  //Debug::printNull();
  return false;
}

bool Data::loadIntegerFromJson(int &variable, String key) {
  //Debug::printStart("Loading integer value from JSON '" + key + "' key");
  if (JSON_DATA.containsKey(key)) {
    int value = JSON_DATA[key].as<int>();
    variable = value;
    //Debug::printLine(String(value));
    return true;
  }
  //Debug::printNull();
  return false;
}

bool Data::loadStringFromJson(String &variable, String key) {
  //Debug::printStart("Loading string value from JSON '" + key + "' key");
  if (JSON_DATA.containsKey(key)) {
    String value = JSON_DATA[key].as<char*>();
    variable = value;
    //Debug::printLine("'" + value + "'");
    return true;
  }
  //Debug::printNull();
  return false;
}

bool Data::loadIsTrueFromJson(String key) {
  //Debug::printStart("Checking true of boolean value from JSON '" + key + "' key");
  bool value = JSON_DATA[key].as<bool>();
  if (value == true) {
    //Debug::printBoolean(true);
    return true;
  }
  //Debug::printBoolean(false);
  return false;
}

bool Data::loadVariablesFromJson() {
  bool status = false;
  if (Data::loadBooleanFromJson(Main::SERVER_MODE, "serverMode")) {
    status = true;
  }
  return status;
}

bool Data::loadConfigFromJson() {
  bool status = false;
  if (Data::loadStringFromJson(Network::SERVER_PASSWORD, "serverPw")) {
    status = true;
  }
  if (Data::loadIntegerFromJson(Pin::BUTTON, "pinButton")) {
    status = true;
  }
  if (Data::loadIntegerFromJson(Pin::RELAY, "pinRelay")) {
    status = true;
  }
  if (Data::loadIntegerFromJson(Pin::LED, "pinLed")) {
    status = true;
  }
  if (Data::loadIntegerFromJson(Pin::EXT_BUTTON, "pinExtButton")) {
    status = true;
  }
  if (Data::loadIntegerFromJson(Pin::EXT_LED, "pinExtLed")) {
    status = true;
  }
  if (Data::loadIntegerFromJson(Pin::LED_1, "pinLed1")) {
    status = true;
  }
  if (Data::loadIntegerFromJson(Pin::LED_2, "pinLed2")) {
    status = true;
  }
  if (Data::loadIntegerFromJson(Pin::LED_3, "pinLed3")) {
    status = true;
  }
  if (Data::loadIntegerFromJson(Pin::TEMP_SENSORS, "pinTemp")) {
    status = true;
  }
  if (Data::loadTempSensorAddressesFromJson("tempAdrs")) {
    status = true;
  }
  return status;
}

bool Data::loadSettingsFromJson() {
  bool status = false;
  if (Data::loadBooleanFromJson(Pin::RELAY_BUTTON_MODE, "buttonMode")) {
    status = true;
  }
  if (Data::loadBooleanFromJson(Pin::RELAY_DEFAULT_STATE, "defaultState")) {
    status = true;
  }
  if (Data::loadIntegerFromJson(Pin::RELAY_MAX_STATE_DELAY, "stateDelay")) {
    status = true;
  }
  if (Data::loadIntegerFromJson(Main::LOOP_DELAY, "loopDelay")) {
    status = true;
  }
  if (Data::loadIntegerFromJson(Pin::DEFAULT_LED_1_VALUE, "defaultLed1")) {
    status = true;
  }
  if (Data::loadIntegerFromJson(Pin::DEFAULT_LED_2_VALUE, "defaultLed2")) {
    status = true;
  }
  if (Data::loadIntegerFromJson(Pin::DEFAULT_LED_3_VALUE, "defaultLed3")) {
    status = true;
  }
  return status;
}

void Data::saveVariablesToJson() {
  JSON_DATA["serverMode"] = Main::SERVER_MODE;
}

void Data::saveConfigToJson() {
  JSON_DATA["serverPw"] = Network::SERVER_PASSWORD;
  JSON_DATA["pinButton"] = Pin::BUTTON;
  JSON_DATA["pinRelay"] = Pin::RELAY;
  JSON_DATA["pinLed"] = Pin::LED;
  JSON_DATA["pinExtButton"] = Pin::EXT_BUTTON;
  JSON_DATA["pinExtLed"] = Pin::EXT_LED;
  JSON_DATA["pinLed1"] = Pin::LED_1;
  JSON_DATA["pinLed2"] = Pin::LED_2;
  JSON_DATA["pinLed3"] = Pin::LED_3;
  JSON_DATA["pinTemp"] = Pin::TEMP_SENSORS;
  saveTempSensorAddressesToJson("tempAdrs");
}

void Data::saveSettingsToJson() {
  JSON_DATA["buttonMode"] = Pin::RELAY_BUTTON_MODE;
  JSON_DATA["defaultState"] = Pin::RELAY_DEFAULT_STATE;
  JSON_DATA["stateDelay"] = Pin::RELAY_MAX_STATE_DELAY;
  JSON_DATA["loopDelay"] = Main::LOOP_DELAY;
  JSON_DATA["defaultLed1"] = Pin::DEFAULT_LED_1_VALUE;
  JSON_DATA["defaultLed2"] = Pin::DEFAULT_LED_2_VALUE;
  JSON_DATA["defaultLed3"] = Pin::DEFAULT_LED_3_VALUE;
}

int Data::loadNetworksFromJson(String key) {
  //Debug::printStart("Loading networks from JSON '" + key + "' key");
  if (JSON_DATA.containsKey(key)) {
    JsonArray networks = JSON_DATA[key].as<JsonArray>();
    int i = 0;
    for (JsonVariant v : networks) {
      if (i >= MAX_NETWORKS_CNT) {
        break;
      }
      String ssid = v["ssid"].as<char*>();
      String pass = v["pw"].as<char*>();
      NETWORKS[i].ssid = ssid;
      NETWORKS[i].password = pass;
      i++;
    }
    if (i > 0) {
      NETWORKS_CNT = i;
      //Debug::printLine(String(NETWORKS_CNT));
      //Debug::printNetworksData("Loaded networks data");
      return NETWORKS_CNT;
    }
  }
  //Debug::printNull();
  return 0;
}

void Data::saveNetworksToJson(String key) {
  //Debug::printStart("Saving networks to JSON '" + key + "' key");
  for (int i = 0; i < NETWORKS_CNT; i++) {
    JSON_DATA[key][i]["ssid"] = NETWORKS[i].ssid;
    JSON_DATA[key][i]["pw"] = NETWORKS[i].password;
  }
  //Debug::printLine("OK");
}

void Data::scanAndSaveNetworksToJson(String key) {
  Network::scan();
  //Debug::printStart("Saving scanned networks to JSON '" + key + "' key");
  for (int i = 0; i < SCANNED_NETWORKS_CNT; i++) {
    JSON_DATA[key][i]["ssid"] = SCANNED_NETWORKS[i].ssid;
    JSON_DATA[key][i]["rssi"] = SCANNED_NETWORKS[i].rssi;
    JSON_DATA[key][i]["bssid"] = SCANNED_NETWORKS[i].bssid;
    JSON_DATA[key][i]["channel"] = SCANNED_NETWORKS[i].channel;
    JSON_DATA[key][i]["encType"] = SCANNED_NETWORKS[i].encryptionType;
  }
  //Debug::printLine("OK");
}

bool Data::loadTempSensorAddressesFromJson(String key) {
  //Debug::printStart("Loading temperature address from JSON '" + key + "' key");
  if (JSON_DATA.containsKey(key)) {
    JsonArray addresses = JSON_DATA[key].as<JsonArray>();
    int i = 0;
    for (JsonVariant u : addresses) {
      JsonArray address = u.as<JsonArray>();
      int j = 0;
      for (JsonVariant v : address) {
        byte value = v.as<byte>();
        TEMPERATURE_SENSOR_ADDRESSES[i][j] = value;
        j++;
      }
      i++;
    }
    if (i > 0) {
      TEMPERATURE_SENSORS_CNT = i;
      //Debug::printLine(String(i));
      return true;
    } else {
      TEMPERATURE_SENSORS_CNT = 0;
      return true;
    }
  }
  //Debug::printNull();
  return false;
}

void Data::saveTempSensorAddressesToJson(String key) {
  //Debug::printStart("Saving temperature address to JSON '" + key + "' key");
  for (int i = 0; i < TEMPERATURE_SENSORS_CNT; i++) {
    for (int j = 0; j < TEMPERATURE_ADDRESS_SIZE; j++) {
      JSON_DATA[key][i][j] = TEMPERATURE_SENSOR_ADDRESSES[i][j];
    }
  }
  //Debug::printLine("OK");
}

bool Data::loadJsonFromFile(String fileName) {
  //Debug::printStart("Loading JSON from '" + fileName + "' file");
  File file = LittleFS.open(fileName, "r");
  if (!file) {
    //Debug::printError("LOAD JSON DATA: Cannot load JSON data from file '" + fileName + "'.");
    return false;
  }
  //Debug::printLine("OK");
  Data::clearJson();
  //Debug::printStart("Deserializing loaded JSON");
  DeserializationError error = deserializeJson(JSON_DATA, file);
  if (error) {
    //Debug::printError("LOAD JSON DATA: Cannot deserialize JSON data from file '" + fileName + "'.");
    //Debug::printError(error.f_str());
    Data::clearJson();
    return false;
  }
  //Debug::printLine("OK");
  //Debug::printJsonData("Loaded JSON data");
  return true;
}

bool Data::saveJsonToFile(String fileName) {
  //Debug::printStart("Saving data to '" + fileName + "' file");
  File file = LittleFS.open(fileName, "w");
  if (!file) {
    //Debug::printError("SAVE JSON DATA: Cannot save JSON data to file '" + fileName + "'.");
    return false;
  }
  //Debug::printLine("OK");
  serializeJson(JSON_DATA, file);
  //Debug::printJsonData("Saved JSON data");
  return true;
}
