/* LOOP CLASS */

void Main::setup() {
  //Debug::init();
  //Debug::printSeparationLine("------------- INITIALIZING -------------");
  Data::init();
  Data::loadDefault();
  Data::loadVariables();
  Data::loadConfig();
  //Pin::TEMP_SENSORS = 14; // TODO REMOVE !!!
  Pin::init();
  Pin::setLedOn();
  Data::loadSettings();
  Pin::changeRelayState(Pin::RELAY_DEFAULT_STATE);
  //Debug::printSetupInfo();
  Data::loadNetworks();
  Network::init();
  Data::clearJson();
  if (!Main::SERVER_MODE) {
    JSON_DATA["fwVer"] = FIRMWARE_VERSION;
    Data::saveConfigToJson();
  }
  Main::sleep(Main::SETUP_DELAY);
  Pin::setLedOff();
  //Debug::printSeparationLine("------- INITIALIZATION COMPLETED -------");
}

void Main::restart() {
  //Debug::printSeparationLine("-------------- RESTARTING --------------");
  //delay(Main::SLEEP_DELAY);
  ESP.restart();
}

void Main::serverNetworkSaveRequest() {
  //Debug::printSeparationLine("--------- NETWORK SAVE REQUEST ---------");
  Data::clearJson();
  String jsonText = "";
  if (SERVER.args() == 1) {
    if (SERVER.argName(0) == "test") {
      String test = SERVER.arg(0);
      //Debug::printLine("Test: '" + test + "'");
      if (test.length() > 0) {
        JSON_DATA["status"] = "OK";
        JSON_DATA["test"] = test;
        serializeJson(JSON_DATA, jsonText);
        SERVER.send(200, "application/json", jsonText);
        return;
      }
    }
  }
  if (SERVER.args() == 2) {
    if (SERVER.argName(0) == "ssid" && SERVER.argName(1) == "pw") {
      String ssid = SERVER.arg(0);
      String password = SERVER.arg(1);
      //Debug::printLine("SSID: '" + ssid + "'");
      //Debug::printLine("Password: '" + password + "'");
      if (ssid.length() > 0) {
        NETWORKS[0].ssid = ssid;
        NETWORKS[0].password = password;
        NETWORKS_CNT = 1;
        Data::saveNetworks();
        JSON_DATA["status"] = "OK";
        serializeJson(JSON_DATA, jsonText);
        SERVER.send(200, "application/json", jsonText);
        delay(Main::SLEEP_DELAY);
        Main::changeModuleMode(false);
        return;
      }
    }
  }
  JSON_DATA["status"] = "ERROR";
  serializeJson(JSON_DATA, jsonText);
  SERVER.send(200, "application/json", jsonText);
}

void Main::serverLoop() {
  SERVER.handleClient();
  Main::protectLongServerLoop();
  Pin::setLedOn();
  delay(Main::SLEEP_DELAY);
  Pin::setLedOff();
  delay(Main::SLEEP_DELAY);
}

void Main::protectLongServerLoop() {
  unsigned long loopTimeDiff = millis() - Main::LAST_LOOP_TIME;
  unsigned long maxLoopDelay = (unsigned long) Main::MAX_SERVER_LOOP_DELAY;
  //Debug::printLine("Loop time difference: " + String(loopTimeDiff) + " ms");
  //Debug::printLine("Maximum loop delay: " + String(maxLoopDelay) + " ms");
  if (loopTimeDiff > maxLoopDelay) {
    Main::changeModuleMode(false);
  }
}

void Main::clientLoop() {
  //Debug::printLine("Served mode is " + String(Main::SERVER_MODE ? "enabled" : "disabled"));
  Pin::protectLongRelayStateChange();
  Main::checkNetworkConnection();
  Main::loopDelay();
  Main::prepareRegularData();

  if (!Network::httpsRequest()) {
    //Debug::printError("LOOP: HTTPS request failed.");
    Network::disconnect();
    Pin::ledBlink(4);
    Data::clearJson();
    return;
  }

  int delay;
  if (!Data::loadIntegerFromJson(delay, "delay")) {
    //Debug::printError("LOOP: Variable 'delay' is missing.");
    Network::disconnect();
    Pin::ledBlink(4);
    Data::clearJson();
    Main::TMP_DELAY = 0;
    return;
  }
  Network::LAST_SUCCESS_REQUEST_TIME = millis();
  if (delay == 0) {
    Data::clearJson();
    Pin::ledBlink(1);
    Main::TMP_DELAY = 0;
    return;
  }
  if (delay == 1) {
    Main::processAll();
    Pin::ledBlink(1);
    Main::TMP_DELAY = Main::MIN_DELAY;
    return;
  }
  if (delay > 1) {
    Pin::ledBlink(1);
    Main::TMP_DELAY = delay >= Main::MIN_DELAY ? delay : Main::MIN_DELAY;
    return;
  }
}

bool Main::checkNetworkConnection() {
  if (!Network::isConnected()) {
    int networkStatus = Network::connect();
    if (networkStatus == Network::CONNECT_STATUS_AP_ERROR) {
      //Debug::printWarning("LOOP: CONNECTING TO NETWORK FAILED");
      Pin::ledBlink(2);
      Main::loopDelay();
      return false;
    }
    if (networkStatus == Network::CONNECT_STATUS_SERVER_ERROR) {
      //Debug::printWarning("LOOP: CONNECTING TO SERVER FAILED");
      Pin::ledBlink(3);
      Main::loopDelay();
      return false;
    }
  }
  return true;
}

void Main::loopDelay() {
  //Debug::printStart("Calculating remaining loop delay");
  unsigned long currentTime = millis();
  if (currentTime < Main::LAST_LOOP_TIME) {
    Main::LAST_LOOP_TIME = currentTime;
  }
  int delay = Main::TMP_DELAY > 0 ? Main::TMP_DELAY : Main::LOOP_DELAY;
  int remLoopDelay = delay - (currentTime - Main::LAST_LOOP_TIME);
  if (remLoopDelay > Main::MAX_CLIENT_LOOP_DELAY) {
    remLoopDelay = Main::MAX_CLIENT_LOOP_DELAY;
  }
  if (remLoopDelay < Main::MIN_DELAY) {
    remLoopDelay = Main::MIN_DELAY;
  }
  //Debug::printLine("OK");
  //Debug::printLine("Current time: " + String(currentTime) + " ms");
  //Debug::printLine("Last loop start time: " + String(Main::LAST_LOOP_TIME) + " ms");
  //Debug::printLine("Required loop/tmp delay: " + String(delay) + " ms");
  //Debug::printLine("Remaining loop delay: " + String(remLoopDelay) + " ms");
  Main::LAST_LOOP_TIME = currentTime + remLoopDelay;
  //Debug::printLine("New last loop start time: " + String(Main::LAST_LOOP_TIME) + " ms");
  Main::sleep(remLoopDelay);
}

void Main::prepareRegularData() {
  if (Pin::RELAY > Pin::NONE) {
    JSON_DATA["relay"] = Pin::RELAY_STATE;
    if (Pin::RELAY_BUTTON_MODE && Pin::RELAY_STATE) {
      Pin::changeRelayState(false);
    }
  }
  JSON_DATA["ssid"] = WiFi.SSID();
  JSON_DATA["rssi"] = WiFi.RSSI();
  //Debug::printLine("Connected network SSID: " + WiFi.SSID());
  //Debug::printLine("Connected network RSSI: " + String(WiFi.RSSI()) + " dBm");
  //Debug::printLine("Local IP: " + WiFi.localIP().toString());
  if (Pin::LED_1 > Pin::NONE) {
    JSON_DATA["led1"] = Pin::LED_1_VALUE;
  }
  if (Pin::LED_2 > Pin::NONE) {
    JSON_DATA["led2"] = Pin::LED_2_VALUE;
  }
  if (Pin::LED_3 > Pin::NONE) {
    JSON_DATA["led3"] = Pin::LED_3_VALUE;
  }
  if (Pin::TEMP_SENSORS > Pin::NONE) {
    //Debug::printLine("Number of sensors: " + String(TEMPERATURE_SENSORS_CNT));
    if (TEMPERATURE_SENSORS_CNT > 0) {
      TEMPERATURE_SENSORS.requestTemperatures();
      for (int i = 0; i < TEMPERATURE_SENSORS_CNT; i++) {
        float temp = TEMPERATURE_SENSORS.getTempC(TEMPERATURE_SENSOR_ADDRESSES[i]);
        //Debug::printLine("Temperature " + String(i) + ": " + String(temp) + " °C");
        JSON_DATA["temps"][i] = temp;
      }
    }
  }
}

void Main::processAll() {
  Main::processRelayState();
  Main::processLedState();
  Main::processUpdate();
  Main::processRestart();
  Main::processConnect();
  bool processSaveConfig = Data::loadConfigFromJson();
  bool processSaveSettings = Data::loadSettingsFromJson();
  bool processSaveNetworks = Data::loadNetworksFromJson("saveNets");
  bool processScanNetworks = Data::loadIsTrueFromJson("scanNets");
  if (processSaveConfig) {
    Data::saveConfig();
    Data::loadConfig();
    Main::restart();
  }
  if (processSaveSettings) {
    Data::saveSettings();
    Data::loadSettings();
  }
  if (processSaveNetworks) {
    Data::saveNetworks();
    Data::loadNetworks();
  }
  Data::clearJson();
  if (processSaveSettings) {
    Data::saveSettingsToJson();
  }
  if (processSaveNetworks) {
    Data::saveNetworksToJson("savedNets");
  }
  if (processScanNetworks) {
    Data::scanAndSaveNetworksToJson("availNets");
  }
}

void Main::processRelayState() {
  bool state = false;
  if (Data::loadBooleanFromJson(state, "relay")) {
    if (Pin::RELAY_BUTTON_MODE) {
      if (state) {
        Pin::pressRelay();
      }
    } else {
      Pin::changeRelayState(state);
    }
  }
}

void Main::processLedState() {
  int led1value = 0;
  int led2value = 0;
  int led3value = 0;
  bool status = false;
  if (Data::loadIntegerFromJson(led1value, "led1")) {
    status = true;
  }
  if (Data::loadIntegerFromJson(led2value, "led2")) {
    status = true;
  }
  if (Data::loadIntegerFromJson(led3value, "led3")) {
    status = true;
  }
  if (status) {
    Pin::changeLedValues(led1value, led2value, led3value);
  }
}

void Main::processUpdate() {
  if (Data::loadStringFromJson(Network::UPDATE_URL, "updateUrl")) {
    Network::httpsUpdate();
  }
}

void Main::processRestart() {
  if (Data::loadIsTrueFromJson("restart")) {
    Main::restart();
  }
}

void Main::processTmpDelay() {
  int tmpDelay = 0;
  if (Data::loadIntegerFromJson(tmpDelay, "tmpDelay")) {
    if (tmpDelay > 0) {
      Main::TMP_DELAY = tmpDelay;
      return;
    }
  }
  Main::TMP_DELAY = 0;
}

void Main::processConnect() {
  //Debug::printStart("Processing connection");
  if (JSON_DATA.containsKey("connSsid")) {
    String ssid = JSON_DATA["connSsid"].as<char*>();
    String password = JSON_DATA["connPw"].as<char*>();
    if (ssid.length() > 0) {
      //Debug::printLine("SSID: " + ssid + ", password: " + password);
      Network::connectTo(ssid, password);
      return;
    }
  }
  //Debug::printNull();
}

void Main::changeModuleMode(bool serverMode) {
  //Debug::printSeparationLine("--------- CHANGING MODULE MODE ---------");
  Main::SERVER_MODE = serverMode;
  Data::saveVariables();
  Main::restart();
}

void Main::sleep(int duration) {
  if (duration > Main::MIN_DELAY && duration < Main::SLEEP_DELAY) {
    delay(duration);
    return;
  }
  int loopCnt = duration / Main::SLEEP_DELAY;
  int remLoopDelay = duration - loopCnt * Main::SLEEP_DELAY;
  //Debug::printLine("Loop cnt: " + String(loopCnt));
  //Debug::printLine("Remain loop delay: " + String(remLoopDelay) + " ms");
  //Debug::printStart("Waiting for " + String(duration) + " ms");
  for (int i = 0; i < loopCnt; i++) {
    if (Main::BREAK_SLEEP) {
      Main::BREAK_SLEEP = false;
      //Debug::printLine("BREAK");
      delay(Main::MIN_DELAY);
      return;
    }
    //Debug::print(".");
    delay(Main::SLEEP_DELAY);
  }
  if (loopCnt) {
    if (remLoopDelay > 0 && remLoopDelay < Main::SLEEP_DELAY) {
      delay(remLoopDelay);
    }
  } else {
    delay(Main::MIN_DELAY);
  }
  //Debug::printLine("OK");
}
