/* PLIX ESP8285 MODULE FILE SYSTEM INITIALIZER */

#include <ESP8266WiFi.h>
#include <WiFiClientSecure.h>
#include <ESP8266httpUpdate.h>
#include <ArduinoJson.h>
#include <LittleFS.h>

const String DEVICE_ID = "XXXXXXXX";
const String DEVICE_TOKEN = "XXXXXXXXXXXXXXXX";

const String HOST = "sm.plix.cz";
const int PORT = 443;
const String UPDATE_URL = "https://sm.plix.cz/api/v1/XXXXXXXXXXXXXXXX/firmware";
const String UPDATE_SSID = "XXXXXXXX";
const String UPDATE_PASS = "XXXXXXXX";

const String SERVER_PASSWORD = "XXXXXXXX";

static const char cert[] PROGMEM = R"EOF(
-----BEGIN CERTIFICATE-----
MIIDSjCCAjKgAwIBAgIQRK+wgNajJ7qJMDmGLvhAazANBgkqhkiG9w0BAQUFADA/
MSQwIgYDVQQKExtEaWdpdGFsIFNpZ25hdHVyZSBUcnVzdCBDby4xFzAVBgNVBAMT
DkRTVCBSb290IENBIFgzMB4XDTAwMDkzMDIxMTIxOVoXDTIxMDkzMDE0MDExNVow
PzEkMCIGA1UEChMbRGlnaXRhbCBTaWduYXR1cmUgVHJ1c3QgQ28uMRcwFQYDVQQD
Ew5EU1QgUm9vdCBDQSBYMzCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEB
AN+v6ZdQCINXtMxiZfaQguzH0yxrMMpb7NnDfcdAwRgUi+DoM3ZJKuM/IUmTrE4O
rz5Iy2Xu/NMhD2XSKtkyj4zl93ewEnu1lcCJo6m67XMuegwGMoOifooUMM0RoOEq
OLl5CjH9UL2AZd+3UWODyOKIYepLYYHsUmu5ouJLGiifSKOeDNoJjj4XLh7dIN9b
xiqKqy69cK3FCxolkHRyxXtqqzTWMIn/5WgTe1QLyNau7Fqckh49ZLOMxt+/yUFw
7BZy1SbsOFU5Q9D8/RhcQPGX69Wam40dutolucbY38EVAjqr2m7xPi71XAicPNaD
aeQQmxkqtilX4+U9m5/wAl0CAwEAAaNCMEAwDwYDVR0TAQH/BAUwAwEB/zAOBgNV
HQ8BAf8EBAMCAQYwHQYDVR0OBBYEFMSnsaR7LHH62+FLkHX/xBVghYkQMA0GCSqG
SIb3DQEBBQUAA4IBAQCjGiybFwBcqR7uKGY3Or+Dxz9LwwmglSBd49lZRNI+DT69
ikugdB/OEIKcdBodfpga3csTS7MgROSR6cz8faXbauX+5v3gTt23ADq1cEmv8uXr
AvHRAosZy5Q6XkjEGB5YGV8eAlrwDPGxrancWYaLbumR9YbK+rlmM6pZW87ipxZz
R8srzJmwN0jP41ZL9c8PDHIyh8bwRLtTcm1D9SZImlJnt1ir/md2cXjbDaJWFBM5
JDGFoqgCWjBH4d1QB7wCCZAA62RjYJsWvIjJEubSfZGL+T0yjWW06XyxV3bqxbYo
Ob8VZRzI9neWagqNdwvYkQsEjgfbKbYK7p2CNTUQ
-----END CERTIFICATE-----
)EOF";
X509List CERT(cert);

const bool SERVER_MODE = false;

const int PIN_BUTTON = 0;
const int PIN_RELAY = 12;
const int PIN_LED = 13;
const int PIN_NONE = -1;

const int RELAY_MAX_STATE_DELAY = 60000;
const int LOOP_DELAY = 10000;

const String DEFAULT_FILE = "/def.json";
const String VARIABLES_FILE = "/var.json";
const String CONFIG_FILE = "/cfg.json";
const String SETTINGS_FILE = "/stg.json";
const String NETWORKS_FILE = "/net.json";

DynamicJsonDocument JSON_DATA(2048);

void setup() {
  Serial.begin(115200);
  delay(1000);
  Serial.println("INITIALIZING PLIX ESP8285 MODULE FILE SYSTEM...");

  Serial.print("Mounting file system: ");
  if (!LittleFS.begin()) {
    Serial.println("ERROR: CANNOT MOUNT FILE SYSTEM");
    return;
  }
  Serial.println("OK");

  Serial.print("Saving JSON to '" + DEFAULT_FILE + "' file: ");
  if (!clearJson()) {
    return;
  }
  JSON_DATA["deviceId"] = DEVICE_ID;
  JSON_DATA["deviceToken"] = DEVICE_TOKEN;
  if (!saveJsonToFile(DEFAULT_FILE)) {
    return;
  }
  Serial.println("OK");

  Serial.print("Saving JSON to '" + VARIABLES_FILE + "' file: ");
  if (!clearJson()) {
    return;
  }
  JSON_DATA["serverMode"] = SERVER_MODE;
  if (!saveJsonToFile(VARIABLES_FILE)) {
    return;
  }
  Serial.println("OK");

  Serial.print("Saving JSON to '" + CONFIG_FILE + "' file: ");
  if (!clearJson()) {
    return;
  }
  const byte tempAdrs[3] = {};
  JSON_DATA["serverPw"] = SERVER_PASSWORD;
  JSON_DATA["pinButton"] = PIN_BUTTON;
  JSON_DATA["pinRelay"] = PIN_RELAY;
  JSON_DATA["pinLed"] = PIN_LED;
  JSON_DATA["pinExtButton"] = PIN_NONE;
  JSON_DATA["pinExtLed"] = PIN_NONE;
  JSON_DATA["pinLed1"] = PIN_NONE;
  JSON_DATA["pinLed2"] = PIN_NONE;
  JSON_DATA["pinLed3"] = PIN_NONE;
  JSON_DATA["pinTemp"] = PIN_NONE;
  JSON_DATA["tempAdrs"] = tempAdrs;
  if (!saveJsonToFile(CONFIG_FILE)) {
    return;
  }
  Serial.println("OK");

  Serial.print("Saving JSON to '" + SETTINGS_FILE + "' file: ");
  if (!clearJson()) {
    return;
  }
  JSON_DATA["buttonMode"] = false;
  JSON_DATA["defaultState"] = false;
  JSON_DATA["stateDelay"] = RELAY_MAX_STATE_DELAY;
  JSON_DATA["loopDelay"] = LOOP_DELAY;
  JSON_DATA["defaultLed1"] = 0;
  JSON_DATA["defaultLed2"] = 0;
  JSON_DATA["defaultLed3"] = 0;
  if (!saveJsonToFile(SETTINGS_FILE)) {
    return;
  }
  Serial.println("OK");

  Serial.print("Saving JSON to '" + NETWORKS_FILE + "' file: ");
  if (!clearJson()) {
    return;
  }
  JSON_DATA["networks"][0]["ssid"] = "Straskovi-wifi";
  JSON_DATA["networks"][0]["pw"] = "vedr58alka";
  if (!saveJsonToFile(NETWORKS_FILE)) {
    return;
  }
  Serial.println("OK");

  Serial.println("FILE SYSTEM SUCCESSFULLY INITIALIZED");

  Serial.println("UPDATING FIRMWARE...");

  Serial.print("Initializing network: ");
  WiFi.mode(WIFI_STA);
  Serial.println("OK");

  if (!networkConnect()) {
    return;
  }

  if (!synchronizeTime()) {
    return;
  }
  
  httpsUpdate();
}

void loop() {
  Serial.println("ERROR");
  delay(10000);
}

bool clearJson() {
  DeserializationError error = deserializeJson(JSON_DATA, "{}");
  if (error) {
    Serial.println("ERROR: CANNOT CLEAR JSON DATA");
    Serial.println(error.f_str());
    return false;
  }
  return true;
}

bool saveJsonToFile(String fileName) {
  File file = LittleFS.open(fileName, "w");
  if (!file) {
    Serial.println("ERROR: CANNOT SAVE DATA TO '" + fileName + "' FILE");
    return false;
  }
  serializeJson(JSON_DATA, file);
  return true;
}

bool networkConnect() {
  Serial.print("Connecting to '" + UPDATE_SSID + "' (time to reset): ");
  WiFi.begin(UPDATE_SSID.c_str(), UPDATE_PASS.c_str());
  for (int i = 0; i < 30000 / 100; i++) {
    const unsigned char status = WiFi.status();
    if (status == WL_CONNECTED) {
      Serial.println("OK");
      return true;
    } else if (status == WL_NO_SSID_AVAIL) {
      Serial.println("ERROR: NETWORK '" + UPDATE_SSID + "' IS NOT AVAILABLE");
      return false;
    } else if (status == WL_CONNECT_FAILED) {
      Serial.println("ERROR: PASSWORD OF '" + UPDATE_SSID + "' NETWORK IS INCORRECT");
      return false;
    }
    Serial.print(".");
    delay(1000);
  }
  Serial.println("CANNOT CONNECT TO '" + UPDATE_SSID + "' NETWORK");
  return false;
}

bool synchronizeTime() {
  Serial.print("Syncing time: ");
  configTime(0, 0, "pool.ntp.org", "time.nist.gov");
  time_t now = time(nullptr);
  for (int i = 0; i < 30000 / 100; i++) {
    if (now < 8 * 3600) {
      Serial.print(".");
      delay(1000);
      now = time(nullptr);
    } else {
      struct tm timeinfo;
      gmtime_r(&now, &timeinfo);
      Serial.print("OK");
      Serial.println(String(asctime(&timeinfo)));
      return true;
    }
  }
  Serial.println("CANNOT SYNC TIME");
  return false;
}

bool httpsUpdate() {
  Serial.print("Updating firmware (time to clear output): ");
  WiFiClientSecure client;
  client.setTrustAnchors(&CERT);
  if (!client.connect(HOST, PORT)) {
    Serial.println("ERROR: CERTIFICATE IS INVALID");
    return false;
  }
  ESPhttpUpdate.update(client, UPDATE_URL);
  Serial.println("ERROR: UPDATING FIRMWARE FAILED");
  return false;
}
