import {ref} from 'vue';
import {Plugins, StatusBarStyle} from '@capacitor/core';
import DeviceInfo from '@/services/DeviceInfo';

const {Storage, StatusBar} = Plugins;
const STORAGE_VARIABLE = 'darkModeEnabled';
const DARK_MODE_BODY_CLASS = 'dark';
const STATUS_BAR_DARK_COLOR = '#1f1f1f';
const STATUS_BAR_LIGHT_COLOR = '#ffffff';

export default abstract class DarkMode {
  private static readonly enabled = ref<boolean>(true);

  public static async initialize() {
    await this.load();
    await this.set();
  }

  public static isEnabled(): boolean {
    return this.enabled.value;
  }

  public static async toggle(value: boolean) {
    this.enabled.value = value;
    await this.set();
    await this.save();
  }

  private static async set() {
    document.body.classList.toggle(DARK_MODE_BODY_CLASS, this.enabled.value);
    if (DeviceInfo.isPlatformAndroid()) {
      if (this.enabled.value) {
        await StatusBar.setStyle({style: StatusBarStyle.Dark});
        await StatusBar.setBackgroundColor({color: STATUS_BAR_DARK_COLOR});
      } else {
        await StatusBar.setStyle({style: StatusBarStyle.Light});
        await StatusBar.setBackgroundColor({color: STATUS_BAR_LIGHT_COLOR});
      }
    }
  }

  private static async load() {
    const ret = await Storage.get({key: STORAGE_VARIABLE});
    if (ret.value) {
      this.enabled.value = JSON.parse(ret.value);
    }
  }

  private static async save() {
    await Storage.set({key: STORAGE_VARIABLE, value: JSON.stringify(this.enabled.value)});
  }
}
