import {ref} from 'vue';
import {Plugins} from '@capacitor/core';
import DarkMode from '@/services/DarkMode';
import GridView from '@/services/GridView';
import FingerPrint from '@/services/FingerPrint';
import Module from '@/services/Module';
import DeviceInfo from '@/services/DeviceInfo';
import Notification from '@/services/Notification';
import Pin from '@/services/Pin';

const {Storage} = Plugins;

export default abstract class Global {
  private static readonly initialized = ref<boolean>(false);

  public static isInitialized(): boolean {
    return this.initialized.value;
  }

  public static async initializeAll() {
    await DeviceInfo.initialize();
    await DarkMode.initialize();
    await GridView.initialize();
    await FingerPrint.initialize();
    await Pin.initialize();
    await Module.initialize();
    this.initialized.value = true;
    await Notification.initialize();
  }

  public static async clearAll() {
    await Storage.clear();
  }
}
