import {ref} from 'vue';
import {Plugins} from '@capacitor/core';

const {Storage} = Plugins;
const STORAGE_VARIABLE = 'gridViewEnabled';

export default abstract class GridView {
  private static readonly enabled = ref<boolean>(true);

  public static async initialize() {
    await this.load();
  }

  public static isEnabled(): boolean {
    return this.enabled.value;
  }

  public static async toggle(value: boolean) {
    this.enabled.value = value;
    await this.save();
  }

  private static async load() {
    const ret = await Storage.get({key: STORAGE_VARIABLE});
    if (ret.value) {
      this.enabled.value = JSON.parse(ret.value);
    }
  }

  private static async save() {
    await Storage.set({key: STORAGE_VARIABLE, value: JSON.stringify(this.enabled.value)});
  }
}
