import DeviceInfo from '@/services/DeviceInfo';
import {Plugins, PushNotification, PushNotificationToken} from '@capacitor/core';
import {alertController} from '@ionic/vue';
import RestApi from '@/services/RestApi';

const {PushNotifications} = Plugins;

export default abstract class Notification {
  public static async initialize() {
    if (DeviceInfo.isPlatformAndroid()) {
      PushNotifications.requestPermission().then((result) => {
        if (result.granted) {
          PushNotifications.register();
        }
      });
      PushNotifications.addListener('registration',
        (token: PushNotificationToken) => {
          RestApi.updateFcm(token.value);
        }
      );
      const showNotification = async (notification: PushNotification) => {
        const alert = await alertController.create({header: notification.title, message: notification.body, buttons: ['OK'], backdropDismiss: false});
        return alert.present();
      }
      PushNotifications.addListener('pushNotificationReceived', showNotification);
    }
  }
}
