import {ref} from 'vue';
import {Plugins} from '@capacitor/core';
import FingerPrint from '@/services/FingerPrint';

const {Storage} = Plugins;
const STORAGE_VARIABLE = 'pinValue';

export default abstract class Pin {
  private static readonly pin = ref<string>('');

  public static async initialize() {
    await this.load();
  }

  public static isEnabled(): boolean {
    return this.pin.value.length > 0;
  }

  public static async change(oldCode: string, newCode: string): Promise<boolean> {
    if (this.pin.value === oldCode) {
      this.pin.value = newCode;
      await this.save();
      return true;
    }
    return false;
  }

  public static verify(value: string): boolean {
    return this.pin.value === value;
  }

  private static async load() {
    const ret = await Storage.get({key: STORAGE_VARIABLE});
    if (ret.value) {
      this.pin.value = JSON.parse(ret.value);
    }
  }

  private static async save() {
    await Storage.set({key: STORAGE_VARIABLE, value: JSON.stringify(this.pin.value)});
    if (!this.pin.value) {
      await FingerPrint.toggle(false);
    }
  }
}
