<?php

namespace Drupal\ban_ip\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class SettingsForm.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ban_ip_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['ban_ip.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('ban_ip.settings');
    $form['threshold'] = [
      '#type' => 'number',
      '#title' => 'Prahová hodnota',
      '#description' => 'Počet porušení pravidel (během intervalu měření), za který bude udělen ban (na dobu trvání).',
      '#default_value' => $config->get('threshold'),
      '#required' => TRUE,
    ];
    $form['measurement_interval'] = [
      '#type' => 'number',
      '#title' => 'Interval měření',
      '#description' => 'Interval (v sekundách), ve kterém se měří počet porušení pravidel.',
      '#default_value' => $config->get('measurement_interval'),
      '#required' => TRUE,
    ];
    $form['duration_time'] = [
      '#type' => 'number',
      '#title' => 'Doba trvání banu',
      '#description' => 'Doba (v sekundách), po kterou trvá ban, než se automaticky zruší (cronem).',
      '#default_value' => $config->get('duration_time'),
      '#required' => TRUE,
    ];
    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save settings'),
      '#button_type' => 'primary',
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $value = $form_state->getValue($key = 'threshold');
    if (!is_numeric($value) || $value <= 0) {
      $form_state->setErrorByName($key, 'Hodnota pole "Práh" není ve správném formátu.');
    }
    $value = $form_state->getValue($key = 'measurement_interval');
    if (!is_numeric($value) || $value <= 0) {
      $form_state->setErrorByName($key, 'Hodnota pole "Interval měření" není ve správném formátu.');
    }
    $value = $form_state->getValue($key = 'duration_time');
    if (!is_numeric($value) || $value <= 0) {
      $form_state->setErrorByName($key, 'Hodnota pole "Doba trvání banu" není ve správném formátu.');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('ban_ip.settings');
    $config->set('threshold', $form_state->getValue('threshold'));
    $config->set('measurement_interval', $form_state->getValue('measurement_interval'));
    $config->set('duration_time', $form_state->getValue('duration_time'));
    $config->save();
    $this->messenger()->addStatus($this->t('Settings saved successfully.'));
  }

}
