<?php

namespace Drupal\plix\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class SettingsForm.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'plix_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['plix.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('plix.settings');
    $form['cron'] = [
      '#type' => 'details',
      '#title' => $this->t('Cron settings'),
      '#open' => TRUE,
    ];
    $cron_urls = $config->get('cron.urls');
    $form['cron']['cron_urls'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Cron urls'),
      '#description' => $this->t('Separate each value with a newline.'),
      '#default_value' => is_array($cron_urls) ? implode(PHP_EOL, $cron_urls) : NULL,
    ];
    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save settings'),
      '#button_type' => 'primary',
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Convert cron urls to array.
    $cron_urls_new = [];
    if ($cron_urls = $form_state->getValue($key = 'cron_urls')) {
      foreach (explode(PHP_EOL, $cron_urls) as $cron_url_raw) {
        if ($cron_url = trim($cron_url_raw)) {
          if (substr($cron_url, 0, 8) != 'https://') {
            $form_state->setErrorByName($key, $this->t('Invalid value.'));
          }
          $cron_urls_new[] = $cron_url;
        }
      }
    }
    $form_state->setValue($key, $cron_urls_new);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('plix.settings');
    $config->set('cron.urls', $form_state->getValue('cron_urls'));
    $config->save();
    $this->messenger()->addStatus($this->t('Settings saved successfully.'));
  }

}
