<?php

namespace Drupal\sm\Form;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\sm\Service\Data\ModuleNodeData;
use Drupal\sm\Service\Database;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class AdminModulesForm.
 */
class AdminModulesForm extends FormBase {

  /**
   * Database service.
   *
   * @var \Drupal\sm\Service\Database
   */
  protected Database $database;

  /**
   * Module node data service.
   *
   * @var \Drupal\sm\Service\Data\ModuleNodeData
   */
  protected ModuleNodeData $moduleNodeData;

  /**
   * Constructor.
   */
  public function __construct(Database $database, ModuleNodeData $module_node_data) {
    $this->database = $database;
    $this->moduleNodeData = $module_node_data;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    /** @var \Drupal\sm\Service\Database $database */
    $database = $container->get('sm.database');
    /** @var \Drupal\sm\Service\Data\ModuleNodeData $module_node_data */
    $module_node_data = $container->get('sm.module_node_data');
    return new static($database, $module_node_data);
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'admin_modules_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $header = [
      'Název',
      'Dostupnost',
      'Poslední spojení',
      'SSID',
      'Síla signálu',
      'Stav',
      'Teploty',
      'Firmware',
      'Operace',
    ];
    $form['table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#empty' => 'Momentálně neexistují žádné moduly.',
    ];

    $current_datetime = new DrupalDateTime('NOW');
    $query = $this->database->nodeStorage->getQuery();
    $query->condition('type', 'module');
    $module_node_ids = $query->execute();
    $module_nodes = $module_node_ids ? $this->database->nodeStorage->loadMultiple($module_node_ids) : NULL;
    foreach ($module_nodes as $i => $module_node) {
      $form['table'][$i][] = [
        '#title' => $module_node->label(),
        '#type' => 'link',
        '#url' => Url::fromRoute('entity.node.canonical', ['node' => $module_node->id()]),
      ];

      $module_last_values = $this->moduleNodeData->getLastValues($module_node);
      $form['table'][$i][]['#markup'] = $this->moduleNodeData->printIsOnline($module_last_values->accessDatetime, $current_datetime);
      $form['table'][$i][]['#markup'] = $this->moduleNodeData->printLastAccess($module_last_values->accessDatetime, $current_datetime);
      $form['table'][$i][]['#markup'] = $module_last_values->ssid;
      $form['table'][$i][]['#markup'] = $this->moduleNodeData->printRssi($module_last_values->rssi);
      $form['table'][$i][]['#markup'] = $this->moduleNodeData->printRelayValue($module_last_values->relay);
      $form['table'][$i][]['#markup'] = $this->moduleNodeData->printTemperatures($module_last_values->temperatures);
      $form['table'][$i][]['#markup'] = $module_last_values->firmwareVersion;

      $form['table'][$i]['operations'] = [
        '#type' => 'operations',
        '#links' => [],
      ];
      $form['table'][$i]['operations']['#links']['edit'] = [
        'title' => t('Edit'),
        'url' => Url::fromRoute('entity.node.edit_form', ['node' => $module_node->id()]),
      ];
      $form['table'][$i]['operations']['#links']['delete'] = [
        'title' => t('Delete'),
        'url' => Url::fromRoute('entity.node.delete_form', ['node' => $module_node->id()]),
      ];
    }

    $form['#attached']['library'][] = 'sm/admin';
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
  }

}
