<?php

namespace Drupal\sm\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\sm\Service\Database;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class GenerateForm.
 */
class GenerateForm extends ConfigFormBase {

  /**
   * Database service.
   *
   * @var \Drupal\sm\Service\Database
   */
  protected Database $database;

  /**
   * Constructor.
   */
  public function __construct(ConfigFactoryInterface $config_factory, Database $database) {
    parent::__construct($config_factory);
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    /** @var \Drupal\Core\Config\ConfigFactoryInterface $config_factory */
    $config_factory = $container->get('config.factory');
    /** @var \Drupal\sm\Service\Database $database */
    $database = $container->get('sm.database');
    return new static($config_factory, $database);
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'sm_generate_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['sm.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['module_cnt'] = [
      '#type' => 'number',
      '#title' => 'Počet nových modulů',
      '#default_value' => 0,
      '#required' => TRUE,
    ];
    $form['temperature_sensor_cnt'] = [
      '#type' => 'number',
      '#title' => 'Počet nových teplotních čidel',
      '#default_value' => 0,
      '#required' => TRUE,
    ];
    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Generate'),
      '#button_type' => 'primary',
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $module_cnt = $form_state->getValue('module_cnt');
    for ($i = 0; $i < $module_cnt; $i++) {
      $this->database->generateModule();
    }
    $temperature_sensor_cnt = $form_state->getValue('temperature_sensor_cnt');
    for ($i = 0; $i < $temperature_sensor_cnt; $i++) {
      $this->database->generateTemperatureSensor();
    }
    $this->messenger()->addStatus('Zařízení byla úspěšně vygenerována.');
  }

}
