<?php

namespace Drupal\sm\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\sm\Service\PushNotification;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class PushNotificationTestForm.
 */
class PushNotificationTestForm extends ConfigFormBase {

  /**
   * Push notification service.
   *
   * @var \Drupal\sm\Service\PushNotification
   */
  protected PushNotification $pushNotification;

  /**
   * Constructor.
   */
  public function __construct(ConfigFactoryInterface $config_factory, PushNotification $push_notification) {
    parent::__construct($config_factory);
    $this->pushNotification = $push_notification;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    /** @var \Drupal\Core\Config\ConfigFactoryInterface $config_factory */
    $config_factory = $container->get('config.factory');
    /** @var \Drupal\sm\Service\PushNotification $push_notification */
    $push_notification = $container->get('sm.push_notification');
    return new static($config_factory, $push_notification);
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'sm_push_notification_test_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['sm.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['to'] = [
      '#type' => 'textfield',
      '#title' => 'To (app FCM token)',
      '#required' => TRUE,
      '#maxlength' => 256,
    ];
    $form['title'] = [
      '#type' => 'textfield',
      '#title' => 'Title',
      '#required' => TRUE,
    ];
    $form['body'] = [
      '#type' => 'textfield',
      '#title' => 'Body',
      '#required' => TRUE,
    ];
    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => 'Odeslat notifikaci',
      '#button_type' => 'primary',
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $result = $this->pushNotification->sendMessage($form_state->getValue('to'), $form_state->getValue('title'), $form_state->getValue('body'));
    if ($result === FALSE) {
      $this->messenger()->addError('Notifikaci se nepodařilo odeslat.');
    }
    $this->messenger()->addStatus('Notifikace byla úspěšně odeslána.');
  }

}
