<?php

namespace Drupal\sm\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class SettingsForm.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'sm_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['sm.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('sm.settings');
    $form['firebase'] = [
      '#type' => 'details',
      '#title' => 'Firebase',
      '#open' => TRUE,
    ];
    $form['firebase']['firebase_server_key'] = [
      '#type' => 'textfield',
      '#title' => 'Server key',
      '#default_value' => $config->get('firebase.server_key'),
      '#required' => TRUE,
      '#maxlength' => 256,
    ];
    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save settings'),
      '#button_type' => 'primary',
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('sm.settings');
    $config->set('firebase.server_key', $form_state->getValue('firebase_server_key'));
    $config->save();
    $this->messenger()->addStatus($this->t('Nastavení bylo úspěšně uloženo.'));
  }

}
