<?php

namespace Drupal\sm\Service;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Class PushNotification.
 */
class PushNotification {

  /**
   * Config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $config;

  /**
   * Constructor.
   */
  public function __construct(ConfigFactoryInterface $config) {
    $this->config = $config;
  }

  /**
   * Sends Firebase cloud message.
   */
  public function sendMessage($fcm_token, $title, $body) {
    $settings = $this->config->get('sm.settings');
    $firebase_server_key = $settings->get('firebase.server_key');
    $headers = [
      'Content-Type:application/json',
      'Authorization:key=' . $firebase_server_key,
    ];
    $data = [
      'to' => $fcm_token,
      'notification' => [
        'title' => $title,
        'body' => $body,
      ],
    ];
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, 'https://fcm.googleapis.com/fcm/send');
    curl_setopt($ch, CURLOPT_POST, TRUE);
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
    curl_setopt($ch, CURLOPT_POSTFIELDS, Json::encode($data));
    $result = curl_exec($ch);
    curl_close($ch);
    if ($result) {
      $data = Json::decode($result);
      if ($data['success']) {
        return TRUE;
      }
    }
    return FALSE;
  }

}
