<?php

namespace Drupal\sm;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\sm\Service\Data\ModuleNodeData;
use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;

/**
 * Extend Drupal's TwigExtension class.
 */
class TwigExtension extends AbstractExtension {

  /**
   * Module node data service.
   *
   * @var \Drupal\sm\Service\Data\ModuleNodeData
   */
  protected ModuleNodeData $moduleNodeData;

  /**
   * Constructor.
   */
  public function __construct(ModuleNodeData $module_node_data) {
    $this->moduleNodeData = $module_node_data;
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return 'sm.twig_extension';
  }

  /**
   * {@inheritdoc}
   */
  public function getFunctions() {
    return [
      new TwigFunction('print_datetime', [$this, 'printDatetime']),
      new TwigFunction('print_datetime_interval', [$this, 'printDatetimeInterval']),
      new TwigFunction('print_encryption_type', [$this, 'printEncryptionType']),
      new TwigFunction('print_led_value', [$this, 'printLedValue']),
      new TwigFunction('print_rssi', [$this, 'printRssi']),
      new TwigFunction('print_status', [$this, 'printStatus']),
    ];
  }

  /**
   * Returns HTML datetime.
   */
  public function printDatetime(?DrupalDateTime $datetime) {
    return $datetime ? $datetime->format(SM_DATETIME_DEFAULT_FORMAT) : NULL;
  }

  /**
   * Returns HTML datetime interval.
   */
  public function printDatetimeInterval(?DrupalDateTime $start_datetime, ?DrupalDateTime $end_datetime, $units_cnt = 4) {
    return $this->moduleNodeData->printDatetimeInterval($start_datetime, $end_datetime, $units_cnt);
  }

  /**
   * Returns HTML encryption type.
   */
  public function printEncryptionType($value) {
    return $this->moduleNodeData->printEncryptionType($value);
  }

  /**
   * Returns HTML LED value.
   */
  public function printLedValue($value) {
    return $this->moduleNodeData->printLedValue($value);
  }

  /**
   * Returns HTML network RSSI.
   */
  public function printRssi(?int $value) {
    return $this->moduleNodeData->printRssi($value);
  }

  /**
   * Returns HTML status.
   */
  public function printStatus(?int $value) {
    return $this->moduleNodeData->printStatus($value);
  }

}
