/**
 * @file
 * Global utilities.
 */
(function ($, Drupal) {
  'use strict';

  $(document).ready(() => {
    refreshButtons();
    refreshMutationObserver();
    refreshWildAnimalAutocomplete();
  });

  function refreshMutationObserver() {
    const target = $('#block-tarabka-bootstrap-content form')[0];
    const options = {
      attributes: false,
      childList: true,
      characterData: false,
      subtree: true,
    };
    let enabled = true;
    const observer = new MutationObserver((mutations) => {
      mutations.some(() => {
        if (enabled) {
          enabled = false;
          refreshButtons();
          setTimeout(() => {
            enabled = true;
          }, 100);
        }
        return true;
      });
    });
    observer.observe(target, options);
  }

  function refreshButtons() {
    generateButtons();
    refreshButtonTooltips();
    refreshButtonListeners();
    refreshWildAnimalAutocomplete();
  }

  function generateButtons() {
    const pasteAllButtonHtml = '<a class="paste-all-button" href="" data-toggle="tooltip" data-placement="bottom" title="Vložit zkopírované řádky"><i class="far fa-clipboard"></i></a>';
    const copyAllButtonHtml = '<a class="copy-all-button" href="" data-toggle="tooltip" data-placement="bottom" title="Kopírovat všechny řádky"><i class="far fa-copy"></i></a>';
    const deleteAllButtonHtml = '<a class="delete-all-button" href="" data-toggle="tooltip" data-placement="bottom" title="Smazat všechny řádky"><i class="far fa-trash-alt"></i></a>';
    const copyButtonHtml = '<a class="copy-button" href="" data-toggle="tooltip" data-placement="bottom" title="Kopírovat řádek"><i class="far fa-copy"></i></a>';
    const deleteButtonHtml = '<a class="delete-button" href="" data-toggle="tooltip" data-placement="bottom" title="Smazat řádek"><i class="far fa-trash-alt"></i></a>';
    const hunterCnt = $('.field--name-field-hunting-permits table tr .field--name-field-hunter').length;
    for (let i = 0; i < hunterCnt; i++) {
      $('[data-drupal-selector="edit-field-hunting-permits-' + i + '-subform"]').append(pasteAllButtonHtml).append(copyAllButtonHtml).append(deleteAllButtonHtml);
      for (let j = 0; j < 7; j++) {
        $('[data-drupal-selector="edit-field-hunting-permits-' + i + '-subform-field-wild-animals-' + j + '-subform"]').append(copyButtonHtml).append(deleteButtonHtml);
      }
    }
  }

  function refreshButtonTooltips() {
    $('[data-toggle="tooltip"]').tooltip({trigger: 'hover'});
  }

  function refreshButtonListeners() {
    $('.paste-all-button').click((event) => {
      event.preventDefault();
      let wildAnimalList = [];
      $('.copy-button.active').each((key, target) => {
        const subformElement = $(target).closest('.paragraphs-subform');
        const kind = subformElement.find('.field--name-field-kind input').val();
        const quantity = subformElement.find('.field--name-field-quantity input').val();
        const sex = subformElement.find('.field--name-field-sex input').val();
        const ageClass = subformElement.find('.field--name-field-age-class input').val();
        if (kind || quantity || sex || ageClass) {
          wildAnimalList.push({kind, quantity, sex, ageClass});
        }
      });
      const copyButtons = $(event.currentTarget).closest('.paragraphs-subform').find('.copy-button');
      copyButtons.each((key, target) => {
        const subformElement = $(target).closest('.paragraphs-subform');
        const kindElement = subformElement.find('.field--name-field-kind input');
        const quantityElement = subformElement.find('.field--name-field-quantity input');
        const sexElement = subformElement.find('.field--name-field-sex input');
        const ageClassElement = subformElement.find('.field--name-field-age-class input');
        if (!kindElement.val() && !quantityElement.val() && !sexElement.val() && !ageClassElement.val()) {
          if (wildAnimalList[0] && (wildAnimalList[0]['kind'] || wildAnimalList[0]['quantity'] || wildAnimalList[0]['sex'] || wildAnimalList[0]['ageClass'])) {
            kindElement.val(wildAnimalList[0]['kind']);
            quantityElement.val(wildAnimalList[0]['quantity']);
            sexElement.val(wildAnimalList[0]['sex']);
            ageClassElement.val(wildAnimalList[0]['ageClass']);
            wildAnimalList.splice(0, 1);
          }
        }
      });
    });
    $('.copy-button').click((event) => {
      event.preventDefault();
      $('.copy-button').removeClass('active');
      $(event.currentTarget).closest('a').addClass('active');
    });
    $('.copy-all-button').click((event) => {
      event.preventDefault();
      $('.copy-button').removeClass('active');
      const copyButtons = $(event.currentTarget).closest('.paragraphs-subform').find('.copy-button');
      copyButtons.each((key, target) => {
        const targetElement = $(target);
        const subformElement = targetElement.closest('.paragraphs-subform');
        const kindElement = subformElement.find('.field--name-field-kind input');
        const quantityElement = subformElement.find('.field--name-field-quantity input');
        const sexElement = subformElement.find('.field--name-field-sex input');
        const ageClassElement = subformElement.find('.field--name-field-age-class input');
        if (kindElement.val() || quantityElement.val() || sexElement.val() || ageClassElement.val()) {
          targetElement.addClass('active');
        }
      });
    });
    $('.delete-button, .delete-all-button').click((event) => {
      event.preventDefault();
      const subformElement = $(event.currentTarget).closest('.paragraphs-subform');
      subformElement.find('.field--name-field-kind input').val('');
      subformElement.find('.field--name-field-quantity input').val('');
      subformElement.find('.field--name-field-sex input').val('');
      subformElement.find('.field--name-field-age-class input').val('');
    });
  }

  function refreshWildAnimalAutocomplete() {
    $('.field--name-field-kind input[type="text"]').autocomplete({
      source: (request, response) => {
        $.ajax({
          dataType: "json",
          type: 'GET',
          url: '/api/tarabka/wild-animals/autocomplete?q=' + request.term,
          success: (data) => {
            response(data);
          },
          error: (error) => {
            console.error(error);
          },
        });
      },
      select: (event, ui) => {
        let data = ui.item;
        if (data['kind'] || data['quantity'] || data['sex'] || data['age_class']) {
          const rowElement = $(event.target).closest('.paragraphs-subform');
          rowElement.find('.field--name-field-quantity input[type="text"]').val(data['quantity']);
          rowElement.find('.field--name-field-sex input[type="text"]').val(data['sex']);
          rowElement.find('.field--name-field-age-class input[type="text"]').val(data['age_class']);
        }
      },
    }).addClass('form-autocomplete');
  }

})(jQuery, Drupal);
