<?php

namespace Drupal\tarabka\Controller;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Controller\ControllerBase;
use Drupal\tarabka\Service\Data\HuntingPermitSeriesNodeData;
use RuntimeException;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class HuntingPermitController.
 */
class HuntingPermitController extends ControllerBase {

  /**
   * Hunter node data service.
   *
   * @var \Drupal\tarabka\Service\Data\HuntingPermitSeriesNodeData
   */
  protected HuntingPermitSeriesNodeData $huntingPermitSeriesNodeData;

  /**
   * Constructor.
   */
  public function __construct(HuntingPermitSeriesNodeData $hunting_permit_series_node_data) {
    $this->huntingPermitSeriesNodeData = $hunting_permit_series_node_data;
  }

  /**
   * Static create function.
   */
  public static function create(ContainerInterface $container) {
    /** @var \Drupal\tarabka\Service\Data\HuntingPermitSeriesNodeData $hunting_permit_series_node_data */
    $hunting_permit_series_node_data = $container->get('tarabka.hunting_permit_series_node_data');
    return new static($hunting_permit_series_node_data);
  }

  /**
   * Returns hunting permits exported in PDF file.
   */
  public function huntingPermitsExport(Request $request) {
    if ($ids = $this->getIds($request)) {
      $this->huntingPermitSeriesNodeData->exportHuntingPermitsToPdf($ids);
    }
    throw new RuntimeException('GENERATE_HUNTING_PERMITS_PDF_FILE_ERROR');
  }

  /**
   * Returns hunting permit list exported in PDF file.
   */
  public function huntingPermitListExport(Request $request) {
    if ($ids = $this->getIds($request)) {
      $this->huntingPermitSeriesNodeData->exportHuntingPermitListToPdf($ids);
    }
    throw new RuntimeException('GENERATE_HUNTING_PERMIT_LIST_PDF_FILE_ERROR');
  }

  /**
   * Returns IDs from request parameter.
   */
  private function getIds(Request $request) {
    $ids_raw = Xss::filter($request->query->get('ids'));
    if (!$ids_raw) {
      return [];
    }
    $ids = explode(',', $ids_raw);
    foreach ($ids as $id) {
      if (!is_numeric($id)) {
        return [];
      }
    }
    return $ids;
  }

}
