<?php

namespace Drupal\tarabka\Controller;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Controller\ControllerBase;
use Drupal\tarabka\Service\Database;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class WildAnimalTermController.
 */
class WildAnimalTermController extends ControllerBase {

  /**
   * Database service.
   *
   * @var \Drupal\tarabka\Service\Database
   */
  protected Database $database;

  /**
   * Constructor.
   */
  public function __construct(Database $database) {
    $this->database = $database;
  }

  /**
   * Static create function.
   */
  public static function create(ContainerInterface $container) {
    /** @var \Drupal\tarabka\Service\Database $database */
    $database = $container->get('tarabka.database');
    return new static($database);
  }

  /**
   * Returns wild animal autocomplete response.
   */
  public function autocomplete(Request $request) {
    $results = [];
    $input = $request->query->get('q');

    if (!$input) {
      return new JsonResponse([]);
    }

    $input = Xss::filter($input);

    $query = $this->database->termStorage->getQuery();
    $query->condition('vid', 'wild_animal');
    $query->condition('name', $input, 'CONTAINS');
    $query->groupBy('tid');
    $query->range(0, 10);

    $term_ids = $query->execute();
    $terms = $term_ids ? $this->database->termStorage->loadMultiple($term_ids) : [];

    foreach ($terms as $term) {
      $results[] = [
        'label' => $term->label(),
        'value' => $term->get('field_kind')->value,
        'quantity' => $term->get('field_quantity')->value,
        'sex' => $term->get('field_sex')->value,
        'age_class' => $term->get('field_age_class')->value,
      ];
    }

    return new JsonResponse($results);
  }

}
