<?php

namespace Drupal\tarabka\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\tarabka\Service\Base;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class SettingsForm.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * Base service.
   *
   * @var \Drupal\tarabka\Service\Base
   */
  protected Base $base;

  /**
   * Constructor.
   */
  public function __construct(ConfigFactoryInterface $config_factory, Base $base) {
    parent::__construct($config_factory);
    $this->base = $base;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    /** @var \Drupal\Core\Config\ConfigFactoryInterface $config_factory */
    $config_factory = $container->get('config.factory');
    /** @var \Drupal\tarabka\Service\Base $base */
    $base = $container->get('tarabka.base');
    return new static($config_factory, $base);
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'tarabka_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['tarabka.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('tarabka.settings');
    $form['hunt'] = [
      '#type' => 'details',
      '#title' => 'Nastavení honitby',
      '#open' => TRUE,
    ];
    $form['hunt']['hunt_name'] = [
      '#type' => 'textfield',
      '#title' => 'Název honitby',
      '#default_value' => $config->get('hunt.name'),
      '#required' => TRUE,
    ];
    $form['hunt']['hunt_user_name'] = [
      '#type' => 'textfield',
      '#title' => 'Uživatel honitby',
      '#default_value' => $config->get('hunt.user.name'),
      '#required' => TRUE,
    ];
    $form['hunt']['hunt_user_residence'] = [
      '#type' => 'textfield',
      '#title' => 'Sídlo uživatele honitby',
      '#default_value' => $config->get('hunt.user.residence'),
      '#required' => TRUE,
    ];
    $form['hunting_permit'] = [
      '#type' => 'details',
      '#title' => 'Nastavení povolenek k lovu',
      '#open' => TRUE,
    ];
    $form['hunting_permit']['hunting_permit_print_signatures'] = [
      '#type' => 'checkbox',
      '#title' => 'Vytisknout také podpis uživatele honitby a mysliveckého hospodáře?',
      '#description' => 'Pozor: Povolenka k lovu nemusí být s vytisknutými podpisy platná!',
      '#default_value' => $config->get('hunting_permit.print_signatures'),
      '#prefix' => '<div class="description-color-danger">',
      '#suffix' => '</div>',
    ];
    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => 'Uložit nastavení',
      '#button_type' => 'primary',
    ];
    $form['#attached']['library'][] = 'tarabka/settings-form';
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $form_state->setValue($key = 'hunt_name', $this->base->removeMultipleWhitespaces($form_state->getValue($key)));
    $form_state->setValue($key = 'hunt_user_name', $this->base->removeMultipleWhitespaces($form_state->getValue($key)));
    $form_state->setValue($key = 'hunt_user_residence', $this->base->removeMultipleWhitespaces($form_state->getValue($key)));
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('tarabka.settings');
    $config->set('hunt.name', $form_state->getValue('hunt_name'));
    $config->set('hunt.user.name', $form_state->getValue('hunt_user_name'));
    $config->set('hunt.user.residence', $form_state->getValue('hunt_user_residence'));
    $config->set('hunting_permit.print_signatures', $form_state->getValue('hunting_permit_print_signatures'));
    $config->save();
    $this->messenger()->addStatus('Nastavení bylo úspěšně uloženo.');
    $form_state->setRedirectUrl(Url::fromRoute('<front>'));
  }

}
