<?php

namespace Drupal\tarabka\Plugin\Block;

use Drupal;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\tarabka\Service\Data\HuntingPermitSeriesNodeData;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides 'Hunting Permit Series List Block'.
 *
 * @Block(
 *   id = "tarabka_hunting_permit_series_list_block",
 *   admin_label = @Translation("Hunting Permit Series List Block"),
 *   category = @Translation("Tarabka"),
 * )
 */
class HuntingPermitSeriesListBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Hunter node data service.
   *
   * @var \Drupal\tarabka\Service\Data\HuntingPermitSeriesNodeData
   */
  protected HuntingPermitSeriesNodeData $huntingPermitSeriesNodeData;

  /**
   * Constructor.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, HuntingPermitSeriesNodeData $hunting_permit_series_node_data) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->huntingPermitSeriesNodeData = $hunting_permit_series_node_data;
  }

  /**
   * Static create function.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    /** @var \Drupal\tarabka\Service\Data\HuntingPermitSeriesNodeData $hunting_permit_series_node_data */
    $hunting_permit_series_node_data = $container->get('tarabka.hunting_permit_series_node_data');
    return new static($configuration, $plugin_id, $plugin_definition, $hunting_permit_series_node_data);
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $results = $this->huntingPermitSeriesNodeData->getHuntingPermitSeriesList();
    return [
      '#theme' => 'hunting_permit_series_list_block',
      '#custom' => ['results' => $results],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function access(AccountInterface $account, $return_as_object = FALSE) {
    if ($account->hasPermission(TARABKA_ADMINISTER_HUNTING_PERMITS_PERM)) {
      return AccessResult::allowed();
    }
    return AccessResult::forbidden();
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    /** @var \Drupal\Core\Entity\EntityInterface $node */
    if ($node = Drupal::routeMatch()->getParameter('node')) {
      if ($node->id() == TARABKA_HUNTING_PERMIT_SERIES_LIST_PAGE_ID) {
        return Cache::mergeTags(parent::getCacheTags(), ['node:' . TARABKA_HUNTING_PERMIT_SERIES_LIST_PAGE_ID]);
      }
    }
    return parent::getCacheTags();
  }

}
