<?php

namespace Drupal\tarabka\Plugin\Block;

use Drupal;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Session\AccountInterface;

/**
 * Provides 'Local Tabs Block'.
 *
 * @Block(
 *   id = "tarabka_local_tabs_block",
 *   admin_label = @Translation("Local Tabs Block"),
 *   category = @Translation("Tarabka"),
 * )
 */
class LocalTabsBlock extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function build() {
    $links = [];
    $route_match = Drupal::routeMatch();
    $route_name = $route_match->getRouteName();
    switch ($route_name) {
      case 'entity.node.canonical':
        /** @var \Drupal\node\NodeInterface $node */
        if ($node = $route_match->getParameter('node')) {
          switch ($node->bundle()) {
            case TARABKA_PAGE_BUNDLE:
              switch ($node->id()) {
                case TARABKA_HUNTING_PERMIT_SERIES_LIST_PAGE_ID:
                  $links[] = [
                    'url' => '/api/tarabka/hunting-permits/export',
                    'text' => 'Vytisknout povolenky',
                    'icon' => 'fas fa-print',
                    'open_in_new_tab' => TRUE,
                    'class' => 'print export-hunting-permits-button',
                  ];
                  $links[] = [
                    'url' => '/api/tarabka/hunting-permits/export-list',
                    'text' => 'Vytisknout seznam povolenek',
                    'icon' => 'fas fa-print',
                    'open_in_new_tab' => TRUE,
                    'class' => 'print export-hunting-permit-list-button',
                  ];
                  $links[] = [
                    'url' => '/node/add/' . TARABKA_HUNTING_PERMIT_SERIES_BUNDLE,
                    'text' => 'Vytvořit novou sérii',
                    'icon' => 'fas fa-plus',
                  ];
                  break;

                case TARABKA_HUNTER_LIST_PAGE_ID:
                  $links[] = [
                    'url' => '/api/tarabka/hunters/export',
                    'text' => 'Vytisknout seznam členů',
                    'icon' => 'fas fa-print',
                    'open_in_new_tab' => TRUE,
                    'class' => 'print',
                  ];
                  $links[] = [
                    'url' => '/node/add/' . TARABKA_HUNTER_BUNDLE,
                    'text' => 'Přidat myslivce',
                    'icon' => 'fas fa-plus',
                  ];
                  break;

                case TARABKA_WILD_ANIMAL_LIST_PAGE_ID:
                  $links[] = [
                    'url' => '/admin/structure/taxonomy/manage/' . TARABKA_WILD_ANIMAL_BUNDLE . '/add',
                    'text' => 'Přidat zvěř',
                    'icon' => 'fas fa-plus',
                  ];
                  break;
              }
              break;

            case TARABKA_HUNTING_PERMIT_SERIES_BUNDLE:
              $links[] = [
                'url' => '/node/' . $node->id() . '/edit',
                'text' => 'Upravit sérii',
                'icon' => 'fas fa-edit',
              ];
              $links[] = [
                'url' => '/node/' . $node->id() . '/delete',
                'text' => 'Smazat sérii',
                'icon' => 'fas fa-trash',
              ];
              break;

            case TARABKA_HUNTER_BUNDLE:
              $links[] = [
                'url' => '/node/' . $node->id() . '/edit',
                'text' => 'Upravit myslivce',
                'icon' => 'fas fa-edit',
              ];
              $links[] = [
                'url' => '/node/' . $node->id() . '/delete',
                'text' => 'Smazat myslivce',
                'icon' => 'fas fa-trash',
              ];
              break;
          }
        }
        break;

      case 'entity.taxonomy_term.canonical':
        /** @var \Drupal\taxonomy\TermInterface $term */
        if ($term = $route_match->getParameter('taxonomy_term')) {
          switch ($term->bundle()) {
            case TARABKA_WILD_ANIMAL_BUNDLE:
              $links[] = [
                'url' => '/taxonomy/term/' . $term->id() . '/edit',
                'text' => 'Upravit zvěř',
                'icon' => 'fas fa-edit',
              ];
              $links[] = [
                'url' => '/taxonomy/term/' . $term->id() . '/delete',
                'text' => 'Smazat zvěř',
                'icon' => 'fas fa-trash',
              ];
              break;
          }
        }
        break;

      case 'entity.user.canonical':
        /** @var \Drupal\user\UserInterface $user */
        if ($user = $route_match->getParameter('user')) {
          $links[] = [
            'url' => '/user/' . $user->id() . '/edit',
            'text' => 'Upravit účet',
            'icon' => 'fas fa-edit',
          ];
          $links[] = [
            'url' => '/user/' . $user->id() . '/cancel',
            'text' => 'Zrušit účet',
            'icon' => 'fas fa-trash',
          ];
        }
        break;
    }
    return [
      '#theme' => 'local_tabs_block',
      '#custom' => ['links' => $links],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function access(AccountInterface $account, $return_as_object = FALSE) {
    if ($account->hasPermission(TARABKA_ADMINISTER_HUNTERS_PERM)) {
      return AccessResult::allowed();
    }
    return AccessResult::forbidden();
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    return Cache::mergeContexts(parent::getCacheContexts(), ['url']);
  }

}
