<?php

namespace Drupal\tarabka\Plugin\Block;

use Drupal;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\tarabka\Service\Data\WildAnimalTermData;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides 'Wild Animal List Block'.
 *
 * @Block(
 *   id = "tarabka_wild_animal_list_block",
 *   admin_label = @Translation("Wild Animal List Block"),
 *   category = @Translation("Tarabka"),
 * )
 */
class WildAnimalListBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Hunter node data service.
   *
   * @var \Drupal\tarabka\Service\Data\WildAnimalTermData
   */
  protected WildAnimalTermData $wildAnimalTermData;

  /**
   * Constructor.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, WildAnimalTermData $wild_animal_term_data) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->wildAnimalTermData = $wild_animal_term_data;
  }

  /**
   * Static create function.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    /** @var \Drupal\tarabka\Service\Data\WildAnimalTermData $wild_animal_term_data */
    $wild_animal_term_data = $container->get('tarabka.wild_animal_term_data');
    return new static($configuration, $plugin_id, $plugin_definition, $wild_animal_term_data);
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $results = $this->wildAnimalTermData->getWildAnimalList();
    return [
      '#theme' => 'wild_animal_list_block',
      '#custom' => ['results' => $results],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function access(AccountInterface $account, $return_as_object = FALSE) {
    if ($account->hasPermission(TARABKA_ADMINISTER_WILD_ANIMALS_PERM)) {
      return AccessResult::allowed();
    }
    return AccessResult::forbidden();
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    /** @var \Drupal\node\NodeInterface $node */
    if ($node = Drupal::routeMatch()->getParameter('node')) {
      if ($node->id() == TARABKA_WILD_ANIMAL_LIST_PAGE_ID) {
        return Cache::mergeTags(parent::getCacheTags(), ['node:' . TARABKA_WILD_ANIMAL_LIST_PAGE_ID]);
      }
    }
    return parent::getCacheTags();
  }

}
