<?php

namespace Drupal\tarabka\Service\Base;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Class DefaultNode.
 */
class DefaultNode {

  /**
   * Controls access to the default node.
   */
  public function access(EntityInterface $entity, $operation, AccountInterface $account) {
    if ($operation == 'view') {
      switch ($entity->id()) {
        case TARABKA_HOME_PAGE_ID:
          if ($account->hasPermission(TARABKA_ADMINISTER_HUNTING_PERMITS_PERM)) {
            $response = new RedirectResponse(Url::fromRoute('entity.node.canonical', ['node' => TARABKA_HUNTING_PERMIT_SERIES_LIST_PAGE_ID])->toString());
            $response->send();
          }
          return AccessResult::allowed();

        case TARABKA_HUNTING_PERMIT_SERIES_LIST_PAGE_ID:
          if ($account->hasPermission(TARABKA_ADMINISTER_HUNTING_PERMITS_PERM)) {
            return AccessResult::allowed();
          }
          return AccessResult::forbidden();

        case TARABKA_HUNTER_LIST_PAGE_ID:
          if ($account->hasPermission(TARABKA_ADMINISTER_HUNTERS_PERM)) {
            return AccessResult::allowed();
          }
          return AccessResult::forbidden();

        case TARABKA_WILD_ANIMAL_LIST_PAGE_ID:
          if ($account->hasPermission(TARABKA_ADMINISTER_WILD_ANIMALS_PERM)) {
            return AccessResult::allowed();
          }
          return AccessResult::forbidden();
      }
    }
    return AccessResult::neutral();
  }

  /**
   * Returns page title.
   */
  public function getPageTitle(&$variables, $route_name) {
    switch ($route_name) {
      case 'entity.user.canonical':
        $variables['title'] = 'Účet';
        return;

      case 'entity.user.edit_form':
        $variables['title'] = 'Upravit účet';
        return;

      case 'user.pass':
        $variables['title'] = 'Obnovení hesla';
        return;
    }
  }

}
