<?php

namespace Drupal\tarabka\Service\Base;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Render\Markup;
use Drupal\node\NodeInterface;

/**
 * Class HunterNode.
 */
class HunterNode {

  /**
   * Runs before saving the hunter node.
   */
  public function beforeSave(NodeInterface $hunter_node) {
    if (!$hunter_node) {
      return;
    }
    $this->regenerateTitle($hunter_node);
  }

  /**
   * Runs after saving the hunter node.
   */
  public function afterSave(NodeInterface $hunter_node) {
    if (!$hunter_node) {
      return;
    }
    $cache_tags = ['node:' . TARABKA_HUNTING_PERMIT_SERIES_LIST_PAGE_ID, 'node:' . TARABKA_HUNTER_LIST_PAGE_ID];
    Cache::invalidateTags($cache_tags);
  }

  /**
   * Returns hunter node page title.
   */
  public function getPageTitle(&$variables, $route_name, NodeInterface $hunter_node = NULL) {
    if ($hunter_node) {
      switch ($route_name) {
        case 'entity.node.edit_form':
          $variables['title'] = Markup::create('Upravit myslivce <em>' . $hunter_node->label() . '</em>');
          return;

        case 'entity.node.delete_form':
          $variables['title'] = Markup::create('Opravdu smazat myslivce <em>' . $hunter_node->label() . '</em> ?');
          return;
      }
    }
    else {
      switch ($route_name) {
        case 'node.add':
          $variables['title'] = 'Přidat nového myslivce';
          return;
      }
    }
  }

  /**
   * Regenerates hunter node title.
   */
  private function regenerateTitle(NodeInterface $hunter_node) {
    $first_name = $hunter_node->get('field_first_name')->value;
    $last_name = $hunter_node->get('field_last_name')->value;
    $hunter_node->setTitle($first_name . ' ' . $last_name);
  }

}
