<?php

namespace Drupal\tarabka\Service\Base;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Render\Markup;
use Drupal\taxonomy\TermInterface;

/**
 * Class WildAnimalTerm.
 */
class WildAnimalTerm {

  /**
   * Runs before saving the wild animal term.
   */
  public function beforeSave(TermInterface $wild_animal_term) {
    if (!$wild_animal_term) {
      return;
    }
    $this->regenerateTitle($wild_animal_term);
  }

  /**
   * Runs after saving the wild animal term.
   */
  public function afterSave(TermInterface $wild_animal_term) {
    if (!$wild_animal_term) {
      return;
    }
    $cache_tags = ['node:' . TARABKA_WILD_ANIMAL_LIST_PAGE_ID];
    Cache::invalidateTags($cache_tags);
  }

  /**
   * Returns wild animal term page title.
   */
  public function getPageTitle(&$variables, $route_name, TermInterface $wild_animal_term = NULL) {
    if ($wild_animal_term) {
      switch ($route_name) {
        case 'entity.taxonomy_term.edit_form':
          $variables['title'] = Markup::create('Upravit zvěř <em>' . $wild_animal_term->label() . '</em>');
          return;

        case 'entity.taxonomy_term.delete_form':
          $variables['title'] = Markup::create('Opravdu smazat zvěř <em>' . $wild_animal_term->label() . '</em> ?');
          return;
      }
    }
    else {
      switch ($route_name) {
        case 'entity.taxonomy_term.add_form':
          $variables['title'] = 'Přidat novou zvěř';
          return;
      }
    }
  }

  /**
   * Regenerates wild animal term title.
   */
  private function regenerateTitle(TermInterface $wild_animal_term) {
    $kind = $wild_animal_term->get('field_kind')->value;
    $quantity = $wild_animal_term->get('field_quantity')->value;
    $sex = $wild_animal_term->get('field_sex')->value;
    $age_class = $wild_animal_term->get('field_age_class')->value;
    $wild_animal_term->setName($kind . ' | ' . $quantity . ' | ' . $sex . ' | ' . $age_class);
  }

}
