<?php

namespace Drupal\tarabka\Service\Data;

use Drupal\tarabka\Service\Database;
use Drupal\taxonomy\TermInterface;

/**
 * Class WildAnimalTermData.
 */
class WildAnimalTermData {

  /**
   * Database service.
   *
   * @var \Drupal\tarabka\Service\Database
   */
  protected Database $database;

  /**
   * Constructor.
   */
  public function __construct(Database $database) {
    $this->database = $database;
  }

  /**
   * Returns hunter data.
   */
  public function getWildAnimal(TermInterface $wild_animal_term) {
    $term_id = $wild_animal_term->id();
    $kind = $wild_animal_term->get('field_kind')->value;
    $quantity = $wild_animal_term->get('field_quantity')->value;
    $sex = $wild_animal_term->get('field_sex')->value;
    $age_class = $wild_animal_term->get('field_age_class')->value;
    return [
      'kind' => $kind,
      'quantity' => $quantity,
      'sex' => $sex,
      'age_class' => $age_class,
      'view_url' => '/zver/' . $term_id,
      'edit_url' => '/taxonomy/term/' . $term_id . '/edit',
      'delete_url' => '/taxonomy/term/' . $term_id . '/delete',
    ];
  }

  /**
   * Returns hunter list data.
   */
  public function getWildAnimalList() {
    $results = [];
    $wild_animal_terms = $this->database->getWildAnimalTerms();
    foreach ($wild_animal_terms as $wild_animal_term) {
      $results[] = $this->getWildAnimal($wild_animal_term);
    }
    return $results;
  }

}
