<?php

namespace Drupal\tarabka\Service\Form;

use Drupal\tarabka\Service\Database;

/**
 * Class HuntingPermitSeriesNodeForm.
 */
class HuntingPermitSeriesNodeForm {

  /**
   * Database service.
   *
   * @var \Drupal\tarabka\Service\Database
   */
  protected Database $database;

  /**
   * Constructor.
   */
  public function __construct(Database $database) {
    $this->database = $database;
  }

  /**
   * Alters hunting permit series add node form.
   */
  public function alterAddForm(array &$form) {
    $this->alterForm($form);
    $form['field_id']['widget'][0]['value']['#default_value'] = $this->database->getHuntingPermitSeriesNodeNextId();
    $i = 0;
    foreach ($this->database->getHunterNodes(TRUE) as $hunter_node) {
      $form['field_hunting_permits']['widget'][$i]['subform']['field_hunter']['widget'][0]['target_id']['#default_value'] = $hunter_node;
      $i++;
    }
  }

  /**
   * Alters hunting permit series edit node form.
   */
  public function alterEditForm(array &$form) {
    $this->alterForm($form);
  }

  /**
   * Alters hunting permit series node delete form.
   */
  public function alterDeleteForm(array &$form) {
    $form['actions']['submit']['#submit'][] = 'Drupal\tarabka\Controller\Form\HuntingPermitSeriesNodeFormController::redirectDeleteForm';
    $form['description']['#markup'] = '<span class="color-danger">Tuto akci nelze vrátit.</span>';
  }

  /**
   * Alters hunting permit series node form.
   */
  private function alterForm(array &$form) {
    $form['#attached']['library'][] = 'tarabka/hunting-permit-series-node-form';
    $form['actions']['submit']['#validate'][] = '::validateForm';
    $form['actions']['submit']['#validate'][] = 'Drupal\tarabka\Controller\Form\HuntingPermitSeriesNodeFormController::validateForm';
    $form['field_hunting_permits']['widget']['add_more']['add_more_button_hunting_permit']['#value'] = 'Přidat další povolenku k lovu';
    $this->fixAutocomplete($form);
  }

  /**
   * Autocomplete fix.
   */
  private function fixAutocomplete(array &$form) {
    $prefix = TARABKA_MODULE_NAME . '_' . TARABKA_HUNTING_PERMIT_SERIES_BUNDLE . '_';
    $form['field_id']['widget'][0]['value']['#attributes']['autocomplete'] = $prefix . 'id';
    $form['field_start_date']['widget'][0]['value']['#attributes']['autocomplete'] = $prefix . 'start_date';
    $form['field_end_date']['widget'][0]['value']['#attributes']['autocomplete'] = $prefix . 'end_date';
  }

}
