<?php

namespace Drupal\tarabka\Service;

use Mpdf\Config\ConfigVariables;
use Mpdf\Config\FontVariables;
use Mpdf\HTMLParserMode;
use Mpdf\Mpdf;
use Mpdf\MpdfException;
use Mpdf\Output\Destination;
use RuntimeException;

/**
 * Class Pdf.
 */
class Pdf {

  /**
   * Generates PDF file.
   */
  public function generate($title, $file_name, $html_markup, $stylesheet_file_name, $options = []) {
    $module_dir_path = DRUPAL_ROOT . '/' . drupal_get_path('module', TARABKA_MODULE_NAME);
    $stylesheet = file_get_contents($module_dir_path . '/css/pdf/' . $stylesheet_file_name . '.css');

    $default_config = (new ConfigVariables())->getDefaults();
    $mpdf_font_dir_paths = $default_config['fontDir'];
    $font_dir_paths = [$module_dir_path . '/fonts'];

    $default_font_config = (new FontVariables())->getDefaults();
    $mpdf_font_data = $default_font_config['fontdata'];
    $font_data = [
      'notoserif' => [
        'R' => 'NotoSerif-Regular.ttf',
        'B' => 'NotoSerif-Bold.ttf',
        'I' => 'NotoSerif-Italic.ttf',
        'BI' => 'NotoSerif-BoldItalic.ttf',
      ],
      'roboto' => [
        'R' => 'Roboto-Regular.ttf',
        'B' => 'Roboto-Bold.ttf',
        'I' => 'Roboto-Italic.ttf',
        'BI' => 'Roboto-BoldItalic.ttf',
      ],
      'robotocondensed' => [
        'R' => 'RobotoCondensed-Regular.ttf',
        'B' => 'RobotoCondensed-Bold.ttf',
        'I' => 'RobotoCondensed-Italic.ttf',
        'BI' => 'RobotoCondensed-BoldItalic.ttf',
      ],
      'arialblack' => [
        'R' => 'Arial-Black.ttf',
        'B' => 'Arial-Black.ttf',
      ],
    ];

    $config = [
      'fontDir' => array_merge($mpdf_font_dir_paths, $font_dir_paths),
      'fontdata' => array_merge($mpdf_font_data, $font_data),
      'default_font' => 'roboto',
      'mode' => 'utf-8',
      'format' => 'A4',
      'orientation' => isset($options['orientation']) ? $options['orientation'] : 'P',
      'margin_left' => 0,
      'margin_right' => 0,
      'margin_top' => 0,
      'margin_bottom' => 0,
      'margin_header' => 0,
      'margin_footer' => 0,
    ];

    try {
      $mpdf = new Mpdf($config);
      $mpdf->SetTitle($title);
      if ($stylesheet) {
        $mpdf->WriteHTML($stylesheet, HTMLParserMode::HEADER_CSS);
      }
      $mpdf->WriteHTML($html_markup, HTMLParserMode::HTML_BODY);
      $mpdf->Output($file_name, Destination::INLINE);
    }
    catch (MpdfException $e) {
      throw new RuntimeException('GENERATE_PDF_FILE_ERROR');
    }
  }

}
