<?php

namespace Drupal\tarabka\Service\Validator;

use Drupal\Core\Form\FormStateInterface;
use Drupal\node\NodeInterface;
use Drupal\tarabka\Service\Database;

/**
 * Class HunterNodeValidator.
 */
class HunterNodeValidator {

  /**
   * Database service.
   *
   * @var \Drupal\tarabka\Service\Database
   */
  protected Database $database;

  /**
   * Default validator service.
   *
   * @var \Drupal\tarabka\Service\Validator\DefaultValidator
   */
  protected DefaultValidator $defaultValidator;

  /**
   * Constructor.
   */
  public function __construct(Database $database, DefaultValidator $default_validator) {
    $this->database = $database;
    $this->defaultValidator = $default_validator;
  }

  /**
   * Validates all hunter node fields.
   */
  public function validateAll(FormStateInterface $form_state) {
    /** @var \Drupal\node\NodeInterface $hunter_node */
    $hunter_node = $form_state->getFormObject()->getEntity();
    $this->validateUniqueness($form_state, $hunter_node);
    $this->defaultValidator->removeMultipleWhitespaces($form_state, 'field_first_name');
    $this->defaultValidator->removeMultipleWhitespaces($form_state, 'field_last_name');
    // Validation of the field_is_member is not required.
    $this->defaultValidator->removeMultipleWhitespaces($form_state, 'field_street');
    $this->defaultValidator->removeMultipleWhitespaces($form_state, 'field_city');
    $this->defaultValidator->validatePostalCode($form_state, 'field_postal_code');
    $this->defaultValidator->validateEmail($form_state, 'field_email');
    $this->defaultValidator->validatePhoneNumber($form_state, 'field_phone_number');
    $this->defaultValidator->removeWhitespaces($form_state, 'field_hunting_license_number');
    $this->defaultValidator->validatePastDate($form_state, 'field_date_of_birth', 'Datum narození nemůže být budoucí.');
    // Validation of the field_gun_license_expiry_date is not required.
  }

  /**
   * Validates uniqueness of hunter node.
   */
  private function validateUniqueness(FormStateInterface $form_state, NodeInterface $hunter_node) {
    $id = (int) $this->defaultValidator->getValueFromField($form_state->getValue('field_id'));
    if (!$id) {
      $form_state->setErrorByName('field_id', 'ID není ve správném formátu.');
      return;
    }
    if ($this->database->countHunterNodes($id, NULL, $hunter_node->id())) {
      $form_state->setErrorByName('field_id', 'ID není unikátní (aktuálně reprezentuje jiného myslivce).');
    }
  }

}
