<?php

namespace Drupal\tarabka\Service\Validator;

use Drupal\Core\Form\FormStateInterface;
use Drupal\tarabka\Service\Database;
use Drupal\taxonomy\TermInterface;

/**
 * Class WildAnimalTermValidator.
 */
class WildAnimalTermValidator {

  /**
   * Database service.
   *
   * @var \Drupal\tarabka\Service\Database
   */
  protected Database $database;

  /**
   * Default validator service.
   *
   * @var \Drupal\tarabka\Service\Validator\DefaultValidator
   */
  protected DefaultValidator $defaultValidator;

  /**
   * Constructor.
   */
  public function __construct(Database $database, DefaultValidator $default_validator) {
    $this->database = $database;
    $this->defaultValidator = $default_validator;
  }

  /**
   * Validates all wild animal term fields.
   */
  public function validateAll(FormStateInterface $form_state) {
    /** @var \Drupal\taxonomy\TermInterface $wild_animal_term */
    $wild_animal_term = $form_state->getFormObject()->getEntity();
    $this->defaultValidator->removeMultipleWhitespaces($form_state, 'field_kind');
    $this->defaultValidator->removeMultipleWhitespaces($form_state, 'field_quantity');
    $this->defaultValidator->removeMultipleWhitespaces($form_state, 'field_sex');
    $this->defaultValidator->removeMultipleWhitespaces($form_state, 'field_age_class');
    $this->validateUniqueness($form_state, $wild_animal_term);
  }

  /**
   * Validates uniqueness of wild animal term.
   */
  public function validateUniqueness(FormStateInterface $form_state, TermInterface $wild_animal_term) {
    $kind = $this->defaultValidator->getValueFromField($form_state->getValue('field_kind'));
    $quantity = $this->defaultValidator->getValueFromField($form_state->getValue('field_quantity'));
    $sex = $this->defaultValidator->getValueFromField($form_state->getValue('field_sex'));
    $age_class = $this->defaultValidator->getValueFromField($form_state->getValue('field_age_class'));
    if (!$kind) {
      $form_state->setErrorByName('field_kind', 'Druh zvěře není ve správném formátu.');
      return;
    }
    if ($this->database->countWildAnimalTerms($kind, $quantity, $sex, $age_class, $wild_animal_term->id())) {
      $form_state->setErrorByName('field_kind', 'Tato zvěř již v seznamu existuje.');
      $form_state->setErrorByName('field_quantity', 'Tato zvěř již v seznamu existuje.');
      $form_state->setErrorByName('field_sex', 'Tato zvěř již v seznamu existuje.');
      $form_state->setErrorByName('field_age_class', 'Tato zvěř již v seznamu existuje.');
    }
  }

}
